#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.0-a53ec6ee1b (http://hl7.org/fhir/StructureDefinition/AdverseEvent) on 2019-05-13.
#  2019, SMART Health IT.


from . import domainresource

class AdverseEvent(domainresource.DomainResource):
    """ Medical care, research study or other healthcare event causing physical
    injury.

    Actual or  potential/avoided event causing unintended physical injury
    resulting from or contributed to by medical care, a research study or other
    healthcare setting factors that requires additional monitoring, treatment,
    or hospitalization, or that results in death.
    """

    resource_type = "AdverseEvent"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.actuality = None
        """ actual | potential.
        Type `str`. """

        self.category = None
        """ product-problem | product-quality | product-use-error | wrong-dose
        | incorrect-prescribing-information | wrong-technique | wrong-
        route-of-administration | wrong-rate | wrong-duration | wrong-time
        | expired-drug | medical-device-use-error | problem-different-
        manufacturer | unsafe-physical-environment.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.contributor = None
        """ Who  was involved in the adverse event or the potential adverse
        event.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.date = None
        """ When the event occurred.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.detected = None
        """ When the event was detected.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.encounter = None
        """ Encounter created as part of.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.event = None
        """ Type of the event itself in relation to the subject.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.identifier = None
        """ Business identifier for the event.
        Type `Identifier` (represented as `dict` in JSON). """

        self.location = None
        """ Location where adverse event occurred.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.outcome = None
        """ resolved | recovering | ongoing | resolvedWithSequelae | fatal |
        unknown.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.recordedDate = None
        """ When the event was recorded.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.recorder = None
        """ Who recorded the adverse event.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.referenceDocument = None
        """ AdverseEvent.referenceDocument.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.resultingCondition = None
        """ Effect on the subject due to this event.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.seriousness = None
        """ Seriousness of the event.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.severity = None
        """ mild | moderate | severe.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.study = None
        """ AdverseEvent.study.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.subject = None
        """ Subject impacted by event.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.subjectMedicalHistory = None
        """ AdverseEvent.subjectMedicalHistory.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.suspectEntity = None
        """ The suspected agent causing the adverse event.
        List of `AdverseEventSuspectEntity` items (represented as `dict` in JSON). """

        super(AdverseEvent, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(AdverseEvent, self).elementProperties()
        js.extend([
            ("actuality", "actuality", str, "code", False, None, True),
            ("category", "category", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("contributor", "contributor", fhirreference.FHIRReference, "Reference", True, None, False),
            ("date", "date", fhirdate.FHIRDate, "dateTime", False, None, False),
            ("detected", "detected", fhirdate.FHIRDate, "dateTime", False, None, False),
            ("encounter", "encounter", fhirreference.FHIRReference, "Reference", False, None, False),
            ("event", "event", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("identifier", "identifier", identifier.Identifier, "Identifier", False, None, False),
            ("location", "location", fhirreference.FHIRReference, "Reference", False, None, False),
            ("outcome", "outcome", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("recordedDate", "recordedDate", fhirdate.FHIRDate, "dateTime", False, None, False),
            ("recorder", "recorder", fhirreference.FHIRReference, "Reference", False, None, False),
            ("referenceDocument", "referenceDocument", fhirreference.FHIRReference, "Reference", True, None, False),
            ("resultingCondition", "resultingCondition", fhirreference.FHIRReference, "Reference", True, None, False),
            ("seriousness", "seriousness", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("severity", "severity", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("study", "study", fhirreference.FHIRReference, "Reference", True, None, False),
            ("subject", "subject", fhirreference.FHIRReference, "Reference", False, None, True),
            ("subjectMedicalHistory", "subjectMedicalHistory", fhirreference.FHIRReference, "Reference", True, None, False),
            ("suspectEntity", "suspectEntity", AdverseEventSuspectEntity, "AdverseEventSuspectEntity", True, None, False),
        ])
        return js


from . import backboneelement

class AdverseEventSuspectEntity(backboneelement.BackboneElement):
    """ The suspected agent causing the adverse event.

    Describes the entity that is suspected to have caused the adverse event.
    """

    resource_type = "AdverseEventSuspectEntity"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.causality = None
        """ Information on the possible cause of the event.
        List of `AdverseEventSuspectEntityCausality` items (represented as `dict` in JSON). """

        self.instance = None
        """ Refers to the specific entity that caused the adverse event.
        Type `FHIRReference` (represented as `dict` in JSON). """

        super(AdverseEventSuspectEntity, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(AdverseEventSuspectEntity, self).elementProperties()
        js.extend([
            ("causality", "causality", AdverseEventSuspectEntityCausality, "AdverseEventSuspectEntityCausality", True, None, False),
            ("instance", "instance", fhirreference.FHIRReference, "Reference", False, None, True),
        ])
        return js


class AdverseEventSuspectEntityCausality(backboneelement.BackboneElement):
    """ Information on the possible cause of the event.
    """

    resource_type = "AdverseEventSuspectEntityCausality"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.assessment = None
        """ Assessment of if the entity caused the event.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.author = None
        """ AdverseEvent.suspectEntity.causalityAuthor.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.method = None
        """ ProbabilityScale | Bayesian | Checklist.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.productRelatedness = None
        """ AdverseEvent.suspectEntity.causalityProductRelatedness.
        Type `str`. """

        super(AdverseEventSuspectEntityCausality, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(AdverseEventSuspectEntityCausality, self).elementProperties()
        js.extend([
            ("assessment", "assessment", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("author", "author", fhirreference.FHIRReference, "Reference", False, None, False),
            ("method", "method", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("productRelatedness", "productRelatedness", str, "string", False, None, False),
        ])
        return js


import sys
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import fhirdate
except ImportError:
    fhirdate = sys.modules[__package__ + '.fhirdate']
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']
try:
    from . import identifier
except ImportError:
    identifier = sys.modules[__package__ + '.identifier']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.0-a53ec6ee1b on 2019-05-13.
#  2019, SMART Health IT.

import os
import pytest
import io
import unittest
import json

from .fixtures import force_bytes
from .. import organizationaffiliation
from ..fhirdate import FHIRDate


@pytest.mark.usefixtures("base_settings")
class OrganizationAffiliationTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("OrganizationAffiliation", js["resourceType"])
        return organizationaffiliation.OrganizationAffiliation(js)
    
    def testOrganizationAffiliation1(self):
        inst = self.instantiate_from("orgrole-example-hie.json")
        self.assertIsNotNone(inst, "Must have instantiated a OrganizationAffiliation instance")
        self.implOrganizationAffiliation1(inst)
        
        js = inst.as_json()
        self.assertEqual("OrganizationAffiliation", js["resourceType"])
        inst2 = organizationaffiliation.OrganizationAffiliation(js)
        self.implOrganizationAffiliation1(inst2)
    
    def implOrganizationAffiliation1(self, inst):
        self.assertTrue(inst.active)
        self.assertEqual(force_bytes(inst.code[0].coding[0].code), force_bytes("member"))
        self.assertEqual(force_bytes(inst.code[0].coding[0].display), force_bytes("Member"))
        self.assertEqual(force_bytes(inst.code[0].coding[0].system), force_bytes("http://hl7.org/fhir/organization-role"))
        self.assertEqual(force_bytes(inst.code[0].text), force_bytes("Hospital member"))
        self.assertEqual(force_bytes(inst.id), force_bytes("orgrole2"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("http://example.org/www.monumentHIE.com"))
        self.assertEqual(force_bytes(inst.identifier[0].type.text), force_bytes("member hospital"))
        self.assertEqual(force_bytes(inst.identifier[0].use), force_bytes("secondary"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("hosp32"))
        self.assertEqual(force_bytes(inst.meta.tag[0].code), force_bytes("HTEST"))
        self.assertEqual(force_bytes(inst.meta.tag[0].display), force_bytes("test health data"))
        self.assertEqual(force_bytes(inst.meta.tag[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/v3-ActReason"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
    
    def testOrganizationAffiliation2(self):
        inst = self.instantiate_from("organizationaffiliation-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a OrganizationAffiliation instance")
        self.implOrganizationAffiliation2(inst)
        
        js = inst.as_json()
        self.assertEqual("OrganizationAffiliation", js["resourceType"])
        inst2 = organizationaffiliation.OrganizationAffiliation(js)
        self.implOrganizationAffiliation2(inst2)
    
    def implOrganizationAffiliation2(self, inst):
        self.assertTrue(inst.active)
        self.assertEqual(force_bytes(inst.code[0].coding[0].code), force_bytes("provider"))
        self.assertEqual(force_bytes(inst.code[0].coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/organization-role"))
        self.assertEqual(force_bytes(inst.id), force_bytes("example"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("http://www.acme.org/practitioners"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("23"))
        self.assertEqual(force_bytes(inst.meta.tag[0].code), force_bytes("HTEST"))
        self.assertEqual(force_bytes(inst.meta.tag[0].display), force_bytes("test health data"))
        self.assertEqual(force_bytes(inst.meta.tag[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/v3-ActReason"))
        self.assertEqual(inst.period.end.date, FHIRDate("2012-03-31").date)
        self.assertEqual(inst.period.end.as_json(), "2012-03-31")
        self.assertEqual(inst.period.start.date, FHIRDate("2012-01-01").date)
        self.assertEqual(inst.period.start.as_json(), "2012-01-01")
        self.assertEqual(force_bytes(inst.specialty[0].coding[0].code), force_bytes("408443003"))
        self.assertEqual(force_bytes(inst.specialty[0].coding[0].display), force_bytes("General medical practice"))
        self.assertEqual(force_bytes(inst.specialty[0].coding[0].system), force_bytes("http://snomed.info/sct"))
        self.assertEqual(force_bytes(inst.telecom[0].system), force_bytes("email"))
        self.assertEqual(force_bytes(inst.telecom[0].use), force_bytes("work"))
        self.assertEqual(force_bytes(inst.telecom[0].value), force_bytes("general.practice@example.org"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
    
    def testOrganizationAffiliation3(self):
        inst = self.instantiate_from("orgrole-example-services.json")
        self.assertIsNotNone(inst, "Must have instantiated a OrganizationAffiliation instance")
        self.implOrganizationAffiliation3(inst)
        
        js = inst.as_json()
        self.assertEqual("OrganizationAffiliation", js["resourceType"])
        inst2 = organizationaffiliation.OrganizationAffiliation(js)
        self.implOrganizationAffiliation3(inst2)
    
    def implOrganizationAffiliation3(self, inst):
        self.assertTrue(inst.active)
        self.assertEqual(force_bytes(inst.code[0].coding[0].code), force_bytes("provider"))
        self.assertEqual(force_bytes(inst.code[0].coding[0].display), force_bytes("Provider"))
        self.assertEqual(force_bytes(inst.code[0].coding[0].system), force_bytes("http://hl7.org/fhir/organization-role"))
        self.assertTrue(inst.code[0].coding[0].userSelected)
        self.assertEqual(force_bytes(inst.code[0].text), force_bytes("Provider of rehabilitation services"))
        self.assertEqual(force_bytes(inst.id), force_bytes("orgrole1"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("http://example.org/www.foundingfathersmemorial.com"))
        self.assertEqual(force_bytes(inst.identifier[0].use), force_bytes("secondary"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("service002"))
        self.assertEqual(force_bytes(inst.meta.tag[0].code), force_bytes("HTEST"))
        self.assertEqual(force_bytes(inst.meta.tag[0].display), force_bytes("test health data"))
        self.assertEqual(force_bytes(inst.meta.tag[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/v3-ActReason"))
        self.assertEqual(inst.period.end.date, FHIRDate("2022-02-01").date)
        self.assertEqual(inst.period.end.as_json(), "2022-02-01")
        self.assertEqual(inst.period.start.date, FHIRDate("2018-02-09").date)
        self.assertEqual(inst.period.start.as_json(), "2018-02-09")
        self.assertEqual(force_bytes(inst.specialty[0].coding[0].code), force_bytes("394602003"))
        self.assertEqual(force_bytes(inst.specialty[0].coding[0].display), force_bytes("Rehabilitation - specialty"))
        self.assertEqual(force_bytes(inst.specialty[0].coding[0].system), force_bytes("http://snomed.info/sct"))
        self.assertEqual(force_bytes(inst.specialty[0].text), force_bytes("Rehabilitation"))
        self.assertEqual(force_bytes(inst.telecom[0].system), force_bytes("phone"))
        self.assertEqual(force_bytes(inst.telecom[0].use), force_bytes("work"))
        self.assertEqual(force_bytes(inst.telecom[0].value), force_bytes("202-109-8765"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.0-a53ec6ee1b (http://hl7.org/fhir/StructureDefinition/DomainResource) on 2019-05-13.
#  2019, SMART Health IT.


from . import resource

class DomainResource(resource.Resource):
    """ A resource with narrative, extensions, and contained resources.

    A resource that includes narrative, extensions, and contained resources.
    """

    resource_type = "DomainResource"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.contained = None
        """ Contained, inline Resources.
        List of `Resource` items (represented as `dict` in JSON). """

        self.extension = None
        """ Additional content defined by implementations.
        List of `Extension` items (represented as `dict` in JSON). """

        self.modifierExtension = None
        """ Extensions that cannot be ignored.
        List of `Extension` items (represented as `dict` in JSON). """

        self.text = None
        """ Text summary of the resource, for human interpretation.
        Type `Narrative` (represented as `dict` in JSON). """

        super(DomainResource, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(DomainResource, self).elementProperties()
        js.extend([
            ("contained", "contained", resource.Resource, "Resource", True, None, False),
            ("extension", "extension", extension.Extension, "Extension", True, None, False),
            ("modifierExtension", "modifierExtension", extension.Extension, "Extension", True, None, False),
            ("text", "text", narrative.Narrative, "Narrative", False, None, False),
        ])
        return js


import sys
try:
    from . import extension
except ImportError:
    extension = sys.modules[__package__ + '.extension']
try:
    from . import narrative
except ImportError:
    narrative = sys.modules[__package__ + '.narrative']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.0-a53ec6ee1b on 2019-05-13.
#  2019, SMART Health IT.

import os
import pytest
import io
import unittest
import json

from .fixtures import force_bytes
from .. import medicinalproductpackaged
from ..fhirdate import FHIRDate


@pytest.mark.usefixtures("base_settings")
class MedicinalProductPackagedTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("MedicinalProductPackaged", js["resourceType"])
        return medicinalproductpackaged.MedicinalProductPackaged(js)
    
    def testMedicinalProductPackaged1(self):
        inst = self.instantiate_from("medicinalproductpackaged-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a MedicinalProductPackaged instance")
        self.implMedicinalProductPackaged1(inst)
        
        js = inst.as_json()
        self.assertEqual("MedicinalProductPackaged", js["resourceType"])
        inst2 = medicinalproductpackaged.MedicinalProductPackaged(js)
        self.implMedicinalProductPackaged1(inst2)
    
    def implMedicinalProductPackaged1(self, inst):
        self.assertEqual(inst.batchIdentifier[0].outerPackaging.period.end.date, FHIRDate("2016-06-06").date)
        self.assertEqual(inst.batchIdentifier[0].outerPackaging.period.end.as_json(), "2016-06-06")
        self.assertEqual(force_bytes(inst.batchIdentifier[0].outerPackaging.system), force_bytes("http://ema.europa.eu/example/baid1"))
        self.assertEqual(force_bytes(inst.batchIdentifier[0].outerPackaging.value), force_bytes("AAF5699"))
        self.assertEqual(force_bytes(inst.description), force_bytes("ALU-PVC/PVDC BLISTERS. CARTONS OF 10 FILM-COATED TABLETS. "))
        self.assertEqual(force_bytes(inst.id), force_bytes("example"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("http://ema.europa.eu/example/pcid"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("{PCID}"))
        self.assertEqual(force_bytes(inst.meta.tag[0].code), force_bytes("HTEST"))
        self.assertEqual(force_bytes(inst.meta.tag[0].display), force_bytes("test health data"))
        self.assertEqual(force_bytes(inst.meta.tag[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/v3-ActReason"))
        self.assertEqual(force_bytes(inst.packageItem[0].material[0].coding[0].code), force_bytes("PVC"))
        self.assertEqual(force_bytes(inst.packageItem[0].material[0].coding[0].system), force_bytes("http://ema.europa.eu/example/packageItemContainerMaterial"))
        self.assertEqual(force_bytes(inst.packageItem[0].material[1].coding[0].code), force_bytes("PVDC"))
        self.assertEqual(force_bytes(inst.packageItem[0].material[1].coding[0].system), force_bytes("http://ema.europa.eu/example/packageItemContainerMaterial"))
        self.assertEqual(force_bytes(inst.packageItem[0].material[2].coding[0].code), force_bytes("alu"))
        self.assertEqual(force_bytes(inst.packageItem[0].material[2].coding[0].system), force_bytes("http://ema.europa.eu/example/packageItemContainerMaterial"))
        self.assertEqual(force_bytes(inst.packageItem[0].packageItem[0].material[0].coding[0].code), force_bytes("Paperboard"))
        self.assertEqual(force_bytes(inst.packageItem[0].packageItem[0].material[0].coding[0].system), force_bytes("http://ema.europa.eu/example/packageItemContainerMaterial"))
        self.assertEqual(force_bytes(inst.packageItem[0].packageItem[0].physicalCharacteristics.height.unit), force_bytes("mm"))
        self.assertEqual(inst.packageItem[0].packageItem[0].physicalCharacteristics.height.value, 125)
        self.assertEqual(force_bytes(inst.packageItem[0].packageItem[0].physicalCharacteristics.width.unit), force_bytes("mm"))
        self.assertEqual(inst.packageItem[0].packageItem[0].physicalCharacteristics.width.value, 45)
        self.assertEqual(force_bytes(inst.packageItem[0].packageItem[0].quantity.unit), force_bytes("1"))
        self.assertEqual(inst.packageItem[0].packageItem[0].quantity.value, 1)
        self.assertEqual(force_bytes(inst.packageItem[0].packageItem[0].shelfLifeStorage[0].period.unit), force_bytes("a"))
        self.assertEqual(inst.packageItem[0].packageItem[0].shelfLifeStorage[0].period.value, 3)
        self.assertEqual(force_bytes(inst.packageItem[0].packageItem[0].shelfLifeStorage[0].specialPrecautionsForStorage[0].coding[0].code), force_bytes("Thismedicinalproductdoesnotrequireanyspecialstoragecondition."))
        self.assertEqual(force_bytes(inst.packageItem[0].packageItem[0].shelfLifeStorage[0].specialPrecautionsForStorage[0].coding[0].system), force_bytes("http://ema.europa.eu/example/specialprecautionsforstorage"))
        self.assertEqual(force_bytes(inst.packageItem[0].packageItem[0].shelfLifeStorage[0].type.coding[0].code), force_bytes("ShelfLifeofPackagedMedicinalProduct"))
        self.assertEqual(force_bytes(inst.packageItem[0].packageItem[0].shelfLifeStorage[0].type.coding[0].system), force_bytes("http://ema.europa.eu/example/shelfLifeTypePlaceHolder"))
        self.assertEqual(force_bytes(inst.packageItem[0].packageItem[0].type.coding[0].code), force_bytes("Blister"))
        self.assertEqual(force_bytes(inst.packageItem[0].packageItem[0].type.coding[0].system), force_bytes("http://ema.europa.eu/example/packageitemcontainertype"))
        self.assertEqual(force_bytes(inst.packageItem[0].physicalCharacteristics.depth.unit), force_bytes("mm"))
        self.assertEqual(inst.packageItem[0].physicalCharacteristics.depth.value, 23.5)
        self.assertEqual(force_bytes(inst.packageItem[0].physicalCharacteristics.height.unit), force_bytes("mm"))
        self.assertEqual(inst.packageItem[0].physicalCharacteristics.height.value, 50)
        self.assertEqual(force_bytes(inst.packageItem[0].physicalCharacteristics.width.unit), force_bytes("mm"))
        self.assertEqual(inst.packageItem[0].physicalCharacteristics.width.value, 136)
        self.assertEqual(force_bytes(inst.packageItem[0].quantity.unit), force_bytes("1"))
        self.assertEqual(inst.packageItem[0].quantity.value, 1)
        self.assertEqual(force_bytes(inst.packageItem[0].type.coding[0].code), force_bytes("Carton"))
        self.assertEqual(force_bytes(inst.packageItem[0].type.coding[0].system), force_bytes("http://ema.europa.eu/example/packageitemcontainertype"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))




# -*- coding: utf-8 -*-
"""Top-level package for fhir.resources."""

__author__ = """Md Nazrul Islam"""

__email__ = 'email2nazrul@gmail.com'

__version__ = "5.0.0b1"

__fhir_version__ = "4.0.0-a53ec6ee1b"


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.0-a53ec6ee1b (http://hl7.org/fhir/StructureDefinition/Invoice) on 2019-05-13.
#  2019, SMART Health IT.


from . import domainresource

class Invoice(domainresource.DomainResource):
    """ Invoice containing ChargeItems from an Account.

    Invoice containing collected ChargeItems from an Account with calculated
    individual and total price for Billing purpose.
    """

    resource_type = "Invoice"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.account = None
        """ Account that is being balanced.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.cancelledReason = None
        """ Reason for cancellation of this Invoice.
        Type `str`. """

        self.date = None
        """ Invoice date / posting date.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.identifier = None
        """ Business Identifier for item.
        List of `Identifier` items (represented as `dict` in JSON). """

        self.issuer = None
        """ Issuing Organization of Invoice.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.lineItem = None
        """ Line items of this Invoice.
        List of `InvoiceLineItem` items (represented as `dict` in JSON). """

        self.note = None
        """ Comments made about the invoice.
        List of `Annotation` items (represented as `dict` in JSON). """

        self.participant = None
        """ Participant in creation of this Invoice.
        List of `InvoiceParticipant` items (represented as `dict` in JSON). """

        self.paymentTerms = None
        """ Payment details.
        Type `str`. """

        self.recipient = None
        """ Recipient of this invoice.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.status = None
        """ draft | issued | balanced | cancelled | entered-in-error.
        Type `str`. """

        self.subject = None
        """ Recipient(s) of goods and services.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.totalGross = None
        """ Gross total of this Invoice.
        Type `Money` (represented as `dict` in JSON). """

        self.totalNet = None
        """ Net total of this Invoice.
        Type `Money` (represented as `dict` in JSON). """

        self.totalPriceComponent = None
        """ Components of Invoice total.
        List of `InvoiceLineItemPriceComponent` items (represented as `dict` in JSON). """

        self.type = None
        """ Type of Invoice.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        super(Invoice, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(Invoice, self).elementProperties()
        js.extend([
            ("account", "account", fhirreference.FHIRReference, "Reference", False, None, False),
            ("cancelledReason", "cancelledReason", str, "string", False, None, False),
            ("date", "date", fhirdate.FHIRDate, "dateTime", False, None, False),
            ("identifier", "identifier", identifier.Identifier, "Identifier", True, None, False),
            ("issuer", "issuer", fhirreference.FHIRReference, "Reference", False, None, False),
            ("lineItem", "lineItem", InvoiceLineItem, "InvoiceLineItem", True, None, False),
            ("note", "note", annotation.Annotation, "Annotation", True, None, False),
            ("participant", "participant", InvoiceParticipant, "InvoiceParticipant", True, None, False),
            ("paymentTerms", "paymentTerms", str, "markdown", False, None, False),
            ("recipient", "recipient", fhirreference.FHIRReference, "Reference", False, None, False),
            ("status", "status", str, "code", False, None, True),
            ("subject", "subject", fhirreference.FHIRReference, "Reference", False, None, False),
            ("totalGross", "totalGross", money.Money, "Money", False, None, False),
            ("totalNet", "totalNet", money.Money, "Money", False, None, False),
            ("totalPriceComponent", "totalPriceComponent", InvoiceLineItemPriceComponent, "InvoiceLineItemPriceComponent", True, None, False),
            ("type", "type", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
        ])
        return js


from . import backboneelement

class InvoiceLineItem(backboneelement.BackboneElement):
    """ Line items of this Invoice.

    Each line item represents one charge for goods and services rendered.
    Details such as date, code and amount are found in the referenced
    ChargeItem resource.
    """

    resource_type = "InvoiceLineItem"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.chargeItemCodeableConcept = None
        """ Reference to ChargeItem containing details of this line item or an
        inline billing code.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.chargeItemReference = None
        """ Reference to ChargeItem containing details of this line item or an
        inline billing code.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.priceComponent = None
        """ Components of total line item price.
        List of `InvoiceLineItemPriceComponent` items (represented as `dict` in JSON). """

        self.sequence = None
        """ Sequence number of line item.
        Type `int`. """

        super(InvoiceLineItem, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(InvoiceLineItem, self).elementProperties()
        js.extend([
            ("chargeItemCodeableConcept", "chargeItemCodeableConcept", codeableconcept.CodeableConcept, "CodeableConcept", False, "chargeItem", True),
            ("chargeItemReference", "chargeItemReference", fhirreference.FHIRReference, "Reference", False, "chargeItem", True),
            ("priceComponent", "priceComponent", InvoiceLineItemPriceComponent, "InvoiceLineItemPriceComponent", True, None, False),
            ("sequence", "sequence", int, "positiveInt", False, None, False),
        ])
        return js


class InvoiceLineItemPriceComponent(backboneelement.BackboneElement):
    """ Components of total line item price.

    The price for a ChargeItem may be calculated as a base price with
    surcharges/deductions that apply in certain conditions. A
    ChargeItemDefinition resource that defines the prices, factors and
    conditions that apply to a billing code is currently under development. The
    priceComponent element can be used to offer transparency to the recipient
    of the Invoice as to how the prices have been calculated.
    """

    resource_type = "InvoiceLineItemPriceComponent"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.amount = None
        """ Monetary amount associated with this component.
        Type `Money` (represented as `dict` in JSON). """

        self.code = None
        """ Code identifying the specific component.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.factor = None
        """ Factor used for calculating this component.
        Type `float`. """

        self.type = None
        """ base | surcharge | deduction | discount | tax | informational.
        Type `str`. """

        super(InvoiceLineItemPriceComponent, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(InvoiceLineItemPriceComponent, self).elementProperties()
        js.extend([
            ("amount", "amount", money.Money, "Money", False, None, False),
            ("code", "code", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("factor", "factor", float, "decimal", False, None, False),
            ("type", "type", str, "code", False, None, True),
        ])
        return js


class InvoiceParticipant(backboneelement.BackboneElement):
    """ Participant in creation of this Invoice.

    Indicates who or what performed or participated in the charged service.
    """

    resource_type = "InvoiceParticipant"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.actor = None
        """ Individual who was involved.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.role = None
        """ Type of involvement in creation of this Invoice.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        super(InvoiceParticipant, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(InvoiceParticipant, self).elementProperties()
        js.extend([
            ("actor", "actor", fhirreference.FHIRReference, "Reference", False, None, True),
            ("role", "role", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
        ])
        return js


import sys
try:
    from . import annotation
except ImportError:
    annotation = sys.modules[__package__ + '.annotation']
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import fhirdate
except ImportError:
    fhirdate = sys.modules[__package__ + '.fhirdate']
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']
try:
    from . import identifier
except ImportError:
    identifier = sys.modules[__package__ + '.identifier']
try:
    from . import money
except ImportError:
    money = sys.modules[__package__ + '.money']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.0-a53ec6ee1b (http://hl7.org/fhir/StructureDefinition/Coverage) on 2019-05-13.
#  2019, SMART Health IT.


from . import domainresource

class Coverage(domainresource.DomainResource):
    """ Insurance or medical plan or a payment agreement.

    Financial instrument which may be used to reimburse or pay for health care
    products and services. Includes both insurance and self-payment.
    """

    resource_type = "Coverage"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.beneficiary = None
        """ Plan beneficiary.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.class_fhir = None
        """ Additional coverage classifications.
        List of `CoverageClass` items (represented as `dict` in JSON). """

        self.contract = None
        """ Contract details.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.costToBeneficiary = None
        """ Patient payments for services/products.
        List of `CoverageCostToBeneficiary` items (represented as `dict` in JSON). """

        self.dependent = None
        """ Dependent number.
        Type `str`. """

        self.identifier = None
        """ Business Identifier for the coverage.
        List of `Identifier` items (represented as `dict` in JSON). """

        self.network = None
        """ Insurer network.
        Type `str`. """

        self.order = None
        """ Relative order of the coverage.
        Type `int`. """

        self.payor = None
        """ Issuer of the policy.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.period = None
        """ Coverage start and end dates.
        Type `Period` (represented as `dict` in JSON). """

        self.policyHolder = None
        """ Owner of the policy.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.relationship = None
        """ Beneficiary relationship to the subscriber.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.status = None
        """ active | cancelled | draft | entered-in-error.
        Type `str`. """

        self.subrogation = None
        """ Reimbursement to insurer.
        Type `bool`. """

        self.subscriber = None
        """ Subscriber to the policy.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.subscriberId = None
        """ ID assigned to the subscriber.
        Type `str`. """

        self.type = None
        """ Coverage category such as medical or accident.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        super(Coverage, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(Coverage, self).elementProperties()
        js.extend([
            ("beneficiary", "beneficiary", fhirreference.FHIRReference, "Reference", False, None, True),
            ("class_fhir", "class", CoverageClass, "CoverageClass", True, None, False),
            ("contract", "contract", fhirreference.FHIRReference, "Reference", True, None, False),
            ("costToBeneficiary", "costToBeneficiary", CoverageCostToBeneficiary, "CoverageCostToBeneficiary", True, None, False),
            ("dependent", "dependent", str, "string", False, None, False),
            ("identifier", "identifier", identifier.Identifier, "Identifier", True, None, False),
            ("network", "network", str, "string", False, None, False),
            ("order", "order", int, "positiveInt", False, None, False),
            ("payor", "payor", fhirreference.FHIRReference, "Reference", True, None, True),
            ("period", "period", period.Period, "Period", False, None, False),
            ("policyHolder", "policyHolder", fhirreference.FHIRReference, "Reference", False, None, False),
            ("relationship", "relationship", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("status", "status", str, "code", False, None, True),
            ("subrogation", "subrogation", bool, "boolean", False, None, False),
            ("subscriber", "subscriber", fhirreference.FHIRReference, "Reference", False, None, False),
            ("subscriberId", "subscriberId", str, "string", False, None, False),
            ("type", "type", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
        ])
        return js


from . import backboneelement

class CoverageClass(backboneelement.BackboneElement):
    """ Additional coverage classifications.

    A suite of underwriter specific classifiers.
    """

    resource_type = "CoverageClass"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.name = None
        """ Human readable description of the type and value.
        Type `str`. """

        self.type = None
        """ Type of class such as 'group' or 'plan'.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.value = None
        """ Value associated with the type.
        Type `str`. """

        super(CoverageClass, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(CoverageClass, self).elementProperties()
        js.extend([
            ("name", "name", str, "string", False, None, False),
            ("type", "type", codeableconcept.CodeableConcept, "CodeableConcept", False, None, True),
            ("value", "value", str, "string", False, None, True),
        ])
        return js


class CoverageCostToBeneficiary(backboneelement.BackboneElement):
    """ Patient payments for services/products.

    A suite of codes indicating the cost category and associated amount which
    have been detailed in the policy and may have been  included on the health
    card.
    """

    resource_type = "CoverageCostToBeneficiary"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.exception = None
        """ Exceptions for patient payments.
        List of `CoverageCostToBeneficiaryException` items (represented as `dict` in JSON). """

        self.type = None
        """ Cost category.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.valueMoney = None
        """ The amount or percentage due from the beneficiary.
        Type `Money` (represented as `dict` in JSON). """

        self.valueQuantity = None
        """ The amount or percentage due from the beneficiary.
        Type `Quantity` (represented as `dict` in JSON). """

        super(CoverageCostToBeneficiary, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(CoverageCostToBeneficiary, self).elementProperties()
        js.extend([
            ("exception", "exception", CoverageCostToBeneficiaryException, "CoverageCostToBeneficiaryException", True, None, False),
            ("type", "type", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("valueMoney", "valueMoney", money.Money, "Money", False, "value", True),
            ("valueQuantity", "valueQuantity", quantity.Quantity, "Quantity", False, "value", True),
        ])
        return js


class CoverageCostToBeneficiaryException(backboneelement.BackboneElement):
    """ Exceptions for patient payments.

    A suite of codes indicating exceptions or reductions to patient costs and
    their effective periods.
    """

    resource_type = "CoverageCostToBeneficiaryException"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.period = None
        """ The effective period of the exception.
        Type `Period` (represented as `dict` in JSON). """

        self.type = None
        """ Exception category.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        super(CoverageCostToBeneficiaryException, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(CoverageCostToBeneficiaryException, self).elementProperties()
        js.extend([
            ("period", "period", period.Period, "Period", False, None, False),
            ("type", "type", codeableconcept.CodeableConcept, "CodeableConcept", False, None, True),
        ])
        return js


import sys
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']
try:
    from . import identifier
except ImportError:
    identifier = sys.modules[__package__ + '.identifier']
try:
    from . import money
except ImportError:
    money = sys.modules[__package__ + '.money']
try:
    from . import period
except ImportError:
    period = sys.modules[__package__ + '.period']
try:
    from . import quantity
except ImportError:
    quantity = sys.modules[__package__ + '.quantity']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.0-a53ec6ee1b on 2019-05-13.
#  2019, SMART Health IT.

import os
import pytest
import io
import unittest
import json

from .fixtures import force_bytes
from .. import claimresponse
from ..fhirdate import FHIRDate


@pytest.mark.usefixtures("base_settings")
class ClaimResponseTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("ClaimResponse", js["resourceType"])
        return claimresponse.ClaimResponse(js)
    
    def testClaimResponse1(self):
        inst = self.instantiate_from("claimresponse-example-2.json")
        self.assertIsNotNone(inst, "Must have instantiated a ClaimResponse instance")
        self.implClaimResponse1(inst)
        
        js = inst.as_json()
        self.assertEqual("ClaimResponse", js["resourceType"])
        inst2 = claimresponse.ClaimResponse(js)
        self.implClaimResponse1(inst2)
    
    def implClaimResponse1(self, inst):
        self.assertEqual(inst.created.date, FHIRDate("2014-08-16").date)
        self.assertEqual(inst.created.as_json(), "2014-08-16")
        self.assertEqual(force_bytes(inst.disposition), force_bytes("Claim could not be processed"))
        self.assertEqual(force_bytes(inst.error[0].code.coding[0].code), force_bytes("a002"))
        self.assertEqual(force_bytes(inst.error[0].code.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/adjudication-error"))
        self.assertEqual(inst.error[0].detailSequence, 2)
        self.assertEqual(inst.error[0].itemSequence, 3)
        self.assertEqual(force_bytes(inst.formCode.coding[0].code), force_bytes("2"))
        self.assertEqual(force_bytes(inst.formCode.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/forms-codes"))
        self.assertEqual(force_bytes(inst.id), force_bytes("R3501"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("http://www.BenefitsInc.com/fhir/remittance"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("R3501"))
        self.assertEqual(force_bytes(inst.meta.tag[0].code), force_bytes("HTEST"))
        self.assertEqual(force_bytes(inst.meta.tag[0].display), force_bytes("test health data"))
        self.assertEqual(force_bytes(inst.meta.tag[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/v3-ActReason"))
        self.assertEqual(force_bytes(inst.outcome), force_bytes("error"))
        self.assertEqual(force_bytes(inst.processNote[0].language.coding[0].code), force_bytes("en-CA"))
        self.assertEqual(force_bytes(inst.processNote[0].language.coding[0].system), force_bytes("urn:ietf:bcp:47"))
        self.assertEqual(inst.processNote[0].number, 1)
        self.assertEqual(force_bytes(inst.processNote[0].text), force_bytes("Invalid claim"))
        self.assertEqual(force_bytes(inst.processNote[0].type), force_bytes("display"))
        self.assertEqual(force_bytes(inst.status), force_bytes("active"))
        self.assertEqual(force_bytes(inst.text.div), force_bytes("<div xmlns=\"http://www.w3.org/1999/xhtml\">A human-readable rendering of the ClaimResponse that demonstrates returning errors</div>"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.type.coding[0].code), force_bytes("oral"))
        self.assertEqual(force_bytes(inst.type.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/claim-type"))
        self.assertEqual(force_bytes(inst.use), force_bytes("claim"))
    
    def testClaimResponse2(self):
        inst = self.instantiate_from("claimresponse-example-additem.json")
        self.assertIsNotNone(inst, "Must have instantiated a ClaimResponse instance")
        self.implClaimResponse2(inst)
        
        js = inst.as_json()
        self.assertEqual("ClaimResponse", js["resourceType"])
        inst2 = claimresponse.ClaimResponse(js)
        self.implClaimResponse2(inst2)
    
    def implClaimResponse2(self, inst):
        self.assertEqual(force_bytes(inst.addItem[0].adjudication[0].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.addItem[0].adjudication[0].amount.value, 100.0)
        self.assertEqual(force_bytes(inst.addItem[0].adjudication[0].category.coding[0].code), force_bytes("eligible"))
        self.assertEqual(force_bytes(inst.addItem[0].adjudication[1].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.addItem[0].adjudication[1].amount.value, 10.0)
        self.assertEqual(force_bytes(inst.addItem[0].adjudication[1].category.coding[0].code), force_bytes("copay"))
        self.assertEqual(force_bytes(inst.addItem[0].adjudication[2].category.coding[0].code), force_bytes("eligpercent"))
        self.assertEqual(inst.addItem[0].adjudication[2].value, 80.0)
        self.assertEqual(force_bytes(inst.addItem[0].adjudication[3].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.addItem[0].adjudication[3].amount.value, 72.0)
        self.assertEqual(force_bytes(inst.addItem[0].adjudication[3].category.coding[0].code), force_bytes("benefit"))
        self.assertEqual(force_bytes(inst.addItem[0].adjudication[3].reason.coding[0].code), force_bytes("ar002"))
        self.assertEqual(force_bytes(inst.addItem[0].adjudication[3].reason.coding[0].display), force_bytes("Plan Limit Reached"))
        self.assertEqual(force_bytes(inst.addItem[0].adjudication[3].reason.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/adjudication-reason"))
        self.assertEqual(inst.addItem[0].itemSequence[0], 1)
        self.assertEqual(force_bytes(inst.addItem[0].modifier[0].coding[0].code), force_bytes("x"))
        self.assertEqual(force_bytes(inst.addItem[0].modifier[0].coding[0].display), force_bytes("None"))
        self.assertEqual(force_bytes(inst.addItem[0].modifier[0].coding[0].system), force_bytes("http://example.org/fhir/modifiers"))
        self.assertEqual(force_bytes(inst.addItem[0].net.currency), force_bytes("USD"))
        self.assertEqual(inst.addItem[0].net.value, 135.57)
        self.assertEqual(inst.addItem[0].noteNumber[0], 101)
        self.assertEqual(force_bytes(inst.addItem[0].productOrService.coding[0].code), force_bytes("1101"))
        self.assertEqual(force_bytes(inst.addItem[0].productOrService.coding[0].system), force_bytes("http://example.org/fhir/oralservicecodes"))
        self.assertEqual(force_bytes(inst.addItem[1].adjudication[0].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.addItem[1].adjudication[0].amount.value, 35.57)
        self.assertEqual(force_bytes(inst.addItem[1].adjudication[0].category.coding[0].code), force_bytes("eligible"))
        self.assertEqual(force_bytes(inst.addItem[1].adjudication[1].category.coding[0].code), force_bytes("eligpercent"))
        self.assertEqual(inst.addItem[1].adjudication[1].value, 80.0)
        self.assertEqual(force_bytes(inst.addItem[1].adjudication[2].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.addItem[1].adjudication[2].amount.value, 28.47)
        self.assertEqual(force_bytes(inst.addItem[1].adjudication[2].category.coding[0].code), force_bytes("benefit"))
        self.assertEqual(inst.addItem[1].itemSequence[0], 1)
        self.assertEqual(force_bytes(inst.addItem[1].net.currency), force_bytes("USD"))
        self.assertEqual(inst.addItem[1].net.value, 35.57)
        self.assertEqual(force_bytes(inst.addItem[1].productOrService.coding[0].code), force_bytes("2141"))
        self.assertEqual(force_bytes(inst.addItem[1].productOrService.coding[0].display), force_bytes("Radiograph, bytewing"))
        self.assertEqual(force_bytes(inst.addItem[1].productOrService.coding[0].system), force_bytes("http://example.org/fhir/oralservicecodes"))
        self.assertEqual(force_bytes(inst.addItem[2].adjudication[0].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.addItem[2].adjudication[0].amount.value, 350.0)
        self.assertEqual(force_bytes(inst.addItem[2].adjudication[0].category.coding[0].code), force_bytes("eligible"))
        self.assertEqual(force_bytes(inst.addItem[2].adjudication[1].category.coding[0].code), force_bytes("eligpercent"))
        self.assertEqual(inst.addItem[2].adjudication[1].value, 80.0)
        self.assertEqual(force_bytes(inst.addItem[2].adjudication[2].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.addItem[2].adjudication[2].amount.value, 270.0)
        self.assertEqual(force_bytes(inst.addItem[2].adjudication[2].category.coding[0].code), force_bytes("benefit"))
        self.assertEqual(inst.addItem[2].detailSequence[0], 2)
        self.assertEqual(inst.addItem[2].itemSequence[0], 3)
        self.assertEqual(force_bytes(inst.addItem[2].modifier[0].coding[0].code), force_bytes("x"))
        self.assertEqual(force_bytes(inst.addItem[2].modifier[0].coding[0].display), force_bytes("None"))
        self.assertEqual(force_bytes(inst.addItem[2].modifier[0].coding[0].system), force_bytes("http://example.org/fhir/modifiers"))
        self.assertEqual(force_bytes(inst.addItem[2].net.currency), force_bytes("USD"))
        self.assertEqual(inst.addItem[2].net.value, 350.0)
        self.assertEqual(inst.addItem[2].noteNumber[0], 101)
        self.assertEqual(force_bytes(inst.addItem[2].productOrService.coding[0].code), force_bytes("expense"))
        self.assertEqual(force_bytes(inst.addItem[2].productOrService.coding[0].system), force_bytes("http://example.org/fhir/oralservicecodes"))
        self.assertEqual(inst.created.date, FHIRDate("2014-08-16").date)
        self.assertEqual(inst.created.as_json(), "2014-08-16")
        self.assertEqual(force_bytes(inst.disposition), force_bytes("Claim settled as per contract."))
        self.assertEqual(force_bytes(inst.id), force_bytes("R3503"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("http://www.BenefitsInc.com/fhir/remittance"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("R3503"))
        self.assertEqual(force_bytes(inst.item[0].adjudication[0].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[0].adjudication[0].amount.value, 0.0)
        self.assertEqual(force_bytes(inst.item[0].adjudication[0].category.coding[0].code), force_bytes("eligible"))
        self.assertEqual(force_bytes(inst.item[0].adjudication[1].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[0].adjudication[1].amount.value, 0.0)
        self.assertEqual(force_bytes(inst.item[0].adjudication[1].category.coding[0].code), force_bytes("benefit"))
        self.assertEqual(inst.item[0].itemSequence, 1)
        self.assertEqual(force_bytes(inst.item[1].adjudication[0].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[1].adjudication[0].amount.value, 105.0)
        self.assertEqual(force_bytes(inst.item[1].adjudication[0].category.coding[0].code), force_bytes("eligible"))
        self.assertEqual(force_bytes(inst.item[1].adjudication[1].category.coding[0].code), force_bytes("eligpercent"))
        self.assertEqual(inst.item[1].adjudication[1].value, 80.0)
        self.assertEqual(force_bytes(inst.item[1].adjudication[2].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[1].adjudication[2].amount.value, 84.0)
        self.assertEqual(force_bytes(inst.item[1].adjudication[2].category.coding[0].code), force_bytes("benefit"))
        self.assertEqual(inst.item[1].itemSequence, 2)
        self.assertEqual(force_bytes(inst.item[2].adjudication[0].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[2].adjudication[0].amount.value, 750.0)
        self.assertEqual(force_bytes(inst.item[2].adjudication[0].category.coding[0].code), force_bytes("eligible"))
        self.assertEqual(force_bytes(inst.item[2].adjudication[1].category.coding[0].code), force_bytes("eligpercent"))
        self.assertEqual(inst.item[2].adjudication[1].value, 80.0)
        self.assertEqual(force_bytes(inst.item[2].adjudication[2].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[2].adjudication[2].amount.value, 600.0)
        self.assertEqual(force_bytes(inst.item[2].adjudication[2].category.coding[0].code), force_bytes("benefit"))
        self.assertEqual(force_bytes(inst.item[2].detail[0].adjudication[0].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[2].detail[0].adjudication[0].amount.value, 750.0)
        self.assertEqual(force_bytes(inst.item[2].detail[0].adjudication[0].category.coding[0].code), force_bytes("eligible"))
        self.assertEqual(force_bytes(inst.item[2].detail[0].adjudication[1].category.coding[0].code), force_bytes("eligpercent"))
        self.assertEqual(inst.item[2].detail[0].adjudication[1].value, 80.0)
        self.assertEqual(force_bytes(inst.item[2].detail[0].adjudication[2].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[2].detail[0].adjudication[2].amount.value, 600.0)
        self.assertEqual(force_bytes(inst.item[2].detail[0].adjudication[2].category.coding[0].code), force_bytes("benefit"))
        self.assertEqual(inst.item[2].detail[0].detailSequence, 1)
        self.assertEqual(force_bytes(inst.item[2].detail[1].adjudication[0].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[2].detail[1].adjudication[0].amount.value, 0.0)
        self.assertEqual(force_bytes(inst.item[2].detail[1].adjudication[0].category.coding[0].code), force_bytes("eligible"))
        self.assertEqual(force_bytes(inst.item[2].detail[1].adjudication[1].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[2].detail[1].adjudication[1].amount.value, 0.0)
        self.assertEqual(force_bytes(inst.item[2].detail[1].adjudication[1].category.coding[0].code), force_bytes("benefit"))
        self.assertEqual(inst.item[2].detail[1].detailSequence, 2)
        self.assertEqual(inst.item[2].itemSequence, 3)
        self.assertEqual(force_bytes(inst.meta.tag[0].code), force_bytes("HTEST"))
        self.assertEqual(force_bytes(inst.meta.tag[0].display), force_bytes("test health data"))
        self.assertEqual(force_bytes(inst.meta.tag[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/v3-ActReason"))
        self.assertEqual(force_bytes(inst.outcome), force_bytes("complete"))
        self.assertEqual(force_bytes(inst.payeeType.coding[0].code), force_bytes("provider"))
        self.assertEqual(force_bytes(inst.payeeType.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/payeetype"))
        self.assertEqual(force_bytes(inst.payment.amount.currency), force_bytes("USD"))
        self.assertEqual(inst.payment.amount.value, 100.47)
        self.assertEqual(inst.payment.date.date, FHIRDate("2014-08-31").date)
        self.assertEqual(inst.payment.date.as_json(), "2014-08-31")
        self.assertEqual(force_bytes(inst.payment.identifier.system), force_bytes("http://www.BenefitsInc.com/fhir/paymentidentifier"))
        self.assertEqual(force_bytes(inst.payment.identifier.value), force_bytes("201408-2-15507"))
        self.assertEqual(force_bytes(inst.payment.type.coding[0].code), force_bytes("complete"))
        self.assertEqual(force_bytes(inst.payment.type.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/ex-paymenttype"))
        self.assertEqual(force_bytes(inst.processNote[0].language.coding[0].code), force_bytes("en-CA"))
        self.assertEqual(force_bytes(inst.processNote[0].language.coding[0].system), force_bytes("urn:ietf:bcp:47"))
        self.assertEqual(inst.processNote[0].number, 101)
        self.assertEqual(force_bytes(inst.processNote[0].text), force_bytes("Package codes are not permitted. Codes replaced by Insurer."))
        self.assertEqual(force_bytes(inst.processNote[0].type), force_bytes("print"))
        self.assertEqual(force_bytes(inst.status), force_bytes("active"))
        self.assertEqual(force_bytes(inst.text.div), force_bytes("<div xmlns=\"http://www.w3.org/1999/xhtml\">A human-readable rendering of the ClaimResponse to Claim Oral Average with additional items</div>"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.total[0].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.total[0].amount.value, 1340.57)
        self.assertEqual(force_bytes(inst.total[0].category.coding[0].code), force_bytes("submitted"))
        self.assertEqual(force_bytes(inst.total[1].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.total[1].amount.value, 1054.47)
        self.assertEqual(force_bytes(inst.total[1].category.coding[0].code), force_bytes("benefit"))
        self.assertEqual(force_bytes(inst.type.coding[0].code), force_bytes("oral"))
        self.assertEqual(force_bytes(inst.type.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/claim-type"))
        self.assertEqual(force_bytes(inst.use), force_bytes("claim"))
    
    def testClaimResponse3(self):
        inst = self.instantiate_from("claimresponse-example-vision-3tier.json")
        self.assertIsNotNone(inst, "Must have instantiated a ClaimResponse instance")
        self.implClaimResponse3(inst)
        
        js = inst.as_json()
        self.assertEqual("ClaimResponse", js["resourceType"])
        inst2 = claimresponse.ClaimResponse(js)
        self.implClaimResponse3(inst2)
    
    def implClaimResponse3(self, inst):
        self.assertEqual(inst.created.date, FHIRDate("2014-08-16").date)
        self.assertEqual(inst.created.as_json(), "2014-08-16")
        self.assertEqual(force_bytes(inst.disposition), force_bytes("Claim settled as per contract."))
        self.assertEqual(force_bytes(inst.id), force_bytes("R3502"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("http://thebenefitcompany.com/claimresponse"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("CR6532875367"))
        self.assertEqual(force_bytes(inst.item[0].adjudication[0].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[0].adjudication[0].amount.value, 235.4)
        self.assertEqual(force_bytes(inst.item[0].adjudication[0].category.coding[0].code), force_bytes("eligible"))
        self.assertEqual(force_bytes(inst.item[0].adjudication[1].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[0].adjudication[1].amount.value, 20.0)
        self.assertEqual(force_bytes(inst.item[0].adjudication[1].category.coding[0].code), force_bytes("copay"))
        self.assertEqual(force_bytes(inst.item[0].adjudication[2].category.coding[0].code), force_bytes("eligpercent"))
        self.assertEqual(inst.item[0].adjudication[2].value, 80.0)
        self.assertEqual(force_bytes(inst.item[0].adjudication[3].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[0].adjudication[3].amount.value, 172.32)
        self.assertEqual(force_bytes(inst.item[0].adjudication[3].category.coding[0].code), force_bytes("benefit"))
        self.assertEqual(force_bytes(inst.item[0].detail[0].adjudication[0].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[0].detail[0].adjudication[0].amount.value, 100.0)
        self.assertEqual(force_bytes(inst.item[0].detail[0].adjudication[0].category.coding[0].code), force_bytes("eligible"))
        self.assertEqual(force_bytes(inst.item[0].detail[0].adjudication[1].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[0].detail[0].adjudication[1].amount.value, 20.0)
        self.assertEqual(force_bytes(inst.item[0].detail[0].adjudication[1].category.coding[0].code), force_bytes("copay"))
        self.assertEqual(force_bytes(inst.item[0].detail[0].adjudication[2].category.coding[0].code), force_bytes("eligpercent"))
        self.assertEqual(inst.item[0].detail[0].adjudication[2].value, 80.0)
        self.assertEqual(force_bytes(inst.item[0].detail[0].adjudication[3].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[0].detail[0].adjudication[3].amount.value, 80.0)
        self.assertEqual(force_bytes(inst.item[0].detail[0].adjudication[3].category.coding[0].code), force_bytes("benefit"))
        self.assertEqual(inst.item[0].detail[0].detailSequence, 1)
        self.assertEqual(inst.item[0].detail[0].noteNumber[0], 1)
        self.assertEqual(force_bytes(inst.item[0].detail[1].adjudication[0].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[0].detail[1].adjudication[0].amount.value, 110.0)
        self.assertEqual(force_bytes(inst.item[0].detail[1].adjudication[0].category.coding[0].code), force_bytes("eligible"))
        self.assertEqual(force_bytes(inst.item[0].detail[1].adjudication[1].category.coding[0].code), force_bytes("eligpercent"))
        self.assertEqual(inst.item[0].detail[1].adjudication[1].value, 80.0)
        self.assertEqual(force_bytes(inst.item[0].detail[1].adjudication[2].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[0].detail[1].adjudication[2].amount.value, 88.0)
        self.assertEqual(force_bytes(inst.item[0].detail[1].adjudication[2].category.coding[0].code), force_bytes("benefit"))
        self.assertEqual(inst.item[0].detail[1].detailSequence, 2)
        self.assertEqual(inst.item[0].detail[1].noteNumber[0], 1)
        self.assertEqual(force_bytes(inst.item[0].detail[1].subDetail[0].adjudication[0].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[0].detail[1].subDetail[0].adjudication[0].amount.value, 60.0)
        self.assertEqual(force_bytes(inst.item[0].detail[1].subDetail[0].adjudication[0].category.coding[0].code), force_bytes("eligible"))
        self.assertEqual(force_bytes(inst.item[0].detail[1].subDetail[0].adjudication[1].category.coding[0].code), force_bytes("eligpercent"))
        self.assertEqual(inst.item[0].detail[1].subDetail[0].adjudication[1].value, 80.0)
        self.assertEqual(force_bytes(inst.item[0].detail[1].subDetail[0].adjudication[2].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[0].detail[1].subDetail[0].adjudication[2].amount.value, 48.0)
        self.assertEqual(force_bytes(inst.item[0].detail[1].subDetail[0].adjudication[2].category.coding[0].code), force_bytes("benefit"))
        self.assertEqual(inst.item[0].detail[1].subDetail[0].noteNumber[0], 1)
        self.assertEqual(inst.item[0].detail[1].subDetail[0].subDetailSequence, 1)
        self.assertEqual(force_bytes(inst.item[0].detail[1].subDetail[1].adjudication[0].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[0].detail[1].subDetail[1].adjudication[0].amount.value, 30.0)
        self.assertEqual(force_bytes(inst.item[0].detail[1].subDetail[1].adjudication[0].category.coding[0].code), force_bytes("eligible"))
        self.assertEqual(force_bytes(inst.item[0].detail[1].subDetail[1].adjudication[1].category.coding[0].code), force_bytes("eligpercent"))
        self.assertEqual(inst.item[0].detail[1].subDetail[1].adjudication[1].value, 80.0)
        self.assertEqual(force_bytes(inst.item[0].detail[1].subDetail[1].adjudication[2].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[0].detail[1].subDetail[1].adjudication[2].amount.value, 24.0)
        self.assertEqual(force_bytes(inst.item[0].detail[1].subDetail[1].adjudication[2].category.coding[0].code), force_bytes("benefit"))
        self.assertEqual(inst.item[0].detail[1].subDetail[1].subDetailSequence, 2)
        self.assertEqual(force_bytes(inst.item[0].detail[1].subDetail[2].adjudication[0].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[0].detail[1].subDetail[2].adjudication[0].amount.value, 10.0)
        self.assertEqual(force_bytes(inst.item[0].detail[1].subDetail[2].adjudication[0].category.coding[0].code), force_bytes("eligible"))
        self.assertEqual(force_bytes(inst.item[0].detail[1].subDetail[2].adjudication[1].category.coding[0].code), force_bytes("eligpercent"))
        self.assertEqual(inst.item[0].detail[1].subDetail[2].adjudication[1].value, 80.0)
        self.assertEqual(force_bytes(inst.item[0].detail[1].subDetail[2].adjudication[2].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[0].detail[1].subDetail[2].adjudication[2].amount.value, 8.0)
        self.assertEqual(force_bytes(inst.item[0].detail[1].subDetail[2].adjudication[2].category.coding[0].code), force_bytes("benefit"))
        self.assertEqual(inst.item[0].detail[1].subDetail[2].noteNumber[0], 1)
        self.assertEqual(inst.item[0].detail[1].subDetail[2].subDetailSequence, 3)
        self.assertEqual(force_bytes(inst.item[0].detail[2].adjudication[0].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[0].detail[2].adjudication[0].amount.value, 200.0)
        self.assertEqual(force_bytes(inst.item[0].detail[2].adjudication[0].category.coding[0].code), force_bytes("eligible"))
        self.assertEqual(force_bytes(inst.item[0].detail[2].adjudication[1].category.coding[0].code), force_bytes("eligpercent"))
        self.assertEqual(inst.item[0].detail[2].adjudication[1].value, 80.0)
        self.assertEqual(force_bytes(inst.item[0].detail[2].adjudication[2].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[0].detail[2].adjudication[2].amount.value, 14.0)
        self.assertEqual(force_bytes(inst.item[0].detail[2].adjudication[2].category.coding[0].code), force_bytes("benefit"))
        self.assertEqual(inst.item[0].detail[2].detailSequence, 3)
        self.assertEqual(inst.item[0].detail[2].noteNumber[0], 1)
        self.assertEqual(inst.item[0].itemSequence, 1)
        self.assertEqual(force_bytes(inst.meta.tag[0].code), force_bytes("HTEST"))
        self.assertEqual(force_bytes(inst.meta.tag[0].display), force_bytes("test health data"))
        self.assertEqual(force_bytes(inst.meta.tag[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/v3-ActReason"))
        self.assertEqual(force_bytes(inst.outcome), force_bytes("complete"))
        self.assertEqual(force_bytes(inst.payeeType.coding[0].code), force_bytes("provider"))
        self.assertEqual(force_bytes(inst.payeeType.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/payeetype"))
        self.assertEqual(force_bytes(inst.payment.adjustment.currency), force_bytes("USD"))
        self.assertEqual(inst.payment.adjustment.value, 75.0)
        self.assertEqual(force_bytes(inst.payment.adjustmentReason.coding[0].code), force_bytes("a002"))
        self.assertEqual(force_bytes(inst.payment.adjustmentReason.coding[0].display), force_bytes("Prior Overpayment"))
        self.assertEqual(force_bytes(inst.payment.adjustmentReason.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/payment-adjustment-reason"))
        self.assertEqual(force_bytes(inst.payment.amount.currency), force_bytes("USD"))
        self.assertEqual(inst.payment.amount.value, 107.0)
        self.assertEqual(inst.payment.date.date, FHIRDate("2014-08-16").date)
        self.assertEqual(inst.payment.date.as_json(), "2014-08-16")
        self.assertEqual(force_bytes(inst.payment.identifier.system), force_bytes("http://thebenefitcompany.com/paymentidentifier"))
        self.assertEqual(force_bytes(inst.payment.identifier.value), force_bytes("201416-123456"))
        self.assertEqual(force_bytes(inst.payment.type.coding[0].code), force_bytes("complete"))
        self.assertEqual(force_bytes(inst.payment.type.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/ex-paymenttype"))
        self.assertEqual(force_bytes(inst.processNote[0].language.coding[0].code), force_bytes("en-CA"))
        self.assertEqual(force_bytes(inst.processNote[0].language.coding[0].system), force_bytes("urn:ietf:bcp:47"))
        self.assertEqual(inst.processNote[0].number, 1)
        self.assertEqual(force_bytes(inst.processNote[0].text), force_bytes("After hours surcharge declined"))
        self.assertEqual(force_bytes(inst.processNote[0].type), force_bytes("display"))
        self.assertEqual(force_bytes(inst.status), force_bytes("active"))
        self.assertEqual(force_bytes(inst.text.div), force_bytes("<div xmlns=\"http://www.w3.org/1999/xhtml\">A human-readable rendering of the ClaimResponse</div>"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.total[0].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.total[0].amount.value, 235.4)
        self.assertEqual(force_bytes(inst.total[0].category.coding[0].code), force_bytes("submitted"))
        self.assertEqual(force_bytes(inst.total[1].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.total[1].amount.value, 182.0)
        self.assertEqual(force_bytes(inst.total[1].category.coding[0].code), force_bytes("benefit"))
        self.assertEqual(force_bytes(inst.type.coding[0].code), force_bytes("vision"))
        self.assertEqual(force_bytes(inst.type.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/claim-type"))
        self.assertEqual(force_bytes(inst.use), force_bytes("claim"))
    
    def testClaimResponse4(self):
        inst = self.instantiate_from("claimresponse-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a ClaimResponse instance")
        self.implClaimResponse4(inst)
        
        js = inst.as_json()
        self.assertEqual("ClaimResponse", js["resourceType"])
        inst2 = claimresponse.ClaimResponse(js)
        self.implClaimResponse4(inst2)
    
    def implClaimResponse4(self, inst):
        self.assertEqual(inst.created.date, FHIRDate("2014-08-16").date)
        self.assertEqual(inst.created.as_json(), "2014-08-16")
        self.assertEqual(force_bytes(inst.disposition), force_bytes("Claim settled as per contract."))
        self.assertEqual(force_bytes(inst.id), force_bytes("R3500"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("http://www.BenefitsInc.com/fhir/remittance"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("R3500"))
        self.assertEqual(force_bytes(inst.item[0].adjudication[0].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[0].adjudication[0].amount.value, 135.57)
        self.assertEqual(force_bytes(inst.item[0].adjudication[0].category.coding[0].code), force_bytes("eligible"))
        self.assertEqual(force_bytes(inst.item[0].adjudication[1].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[0].adjudication[1].amount.value, 10.0)
        self.assertEqual(force_bytes(inst.item[0].adjudication[1].category.coding[0].code), force_bytes("copay"))
        self.assertEqual(force_bytes(inst.item[0].adjudication[2].category.coding[0].code), force_bytes("eligpercent"))
        self.assertEqual(inst.item[0].adjudication[2].value, 80.0)
        self.assertEqual(force_bytes(inst.item[0].adjudication[3].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.item[0].adjudication[3].amount.value, 90.47)
        self.assertEqual(force_bytes(inst.item[0].adjudication[3].category.coding[0].code), force_bytes("benefit"))
        self.assertEqual(force_bytes(inst.item[0].adjudication[3].reason.coding[0].code), force_bytes("ar002"))
        self.assertEqual(force_bytes(inst.item[0].adjudication[3].reason.coding[0].display), force_bytes("Plan Limit Reached"))
        self.assertEqual(force_bytes(inst.item[0].adjudication[3].reason.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/adjudication-reason"))
        self.assertEqual(inst.item[0].itemSequence, 1)
        self.assertEqual(force_bytes(inst.meta.tag[0].code), force_bytes("HTEST"))
        self.assertEqual(force_bytes(inst.meta.tag[0].display), force_bytes("test health data"))
        self.assertEqual(force_bytes(inst.meta.tag[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/v3-ActReason"))
        self.assertEqual(force_bytes(inst.outcome), force_bytes("complete"))
        self.assertEqual(force_bytes(inst.payeeType.coding[0].code), force_bytes("provider"))
        self.assertEqual(force_bytes(inst.payeeType.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/payeetype"))
        self.assertEqual(force_bytes(inst.payment.amount.currency), force_bytes("USD"))
        self.assertEqual(inst.payment.amount.value, 100.47)
        self.assertEqual(inst.payment.date.date, FHIRDate("2014-08-31").date)
        self.assertEqual(inst.payment.date.as_json(), "2014-08-31")
        self.assertEqual(force_bytes(inst.payment.identifier.system), force_bytes("http://www.BenefitsInc.com/fhir/paymentidentifier"))
        self.assertEqual(force_bytes(inst.payment.identifier.value), force_bytes("201408-2-1569478"))
        self.assertEqual(force_bytes(inst.payment.type.coding[0].code), force_bytes("complete"))
        self.assertEqual(force_bytes(inst.payment.type.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/ex-paymenttype"))
        self.assertEqual(force_bytes(inst.status), force_bytes("active"))
        self.assertEqual(force_bytes(inst.subType.coding[0].code), force_bytes("emergency"))
        self.assertEqual(force_bytes(inst.subType.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/ex-claimsubtype"))
        self.assertEqual(force_bytes(inst.text.div), force_bytes("<div xmlns=\"http://www.w3.org/1999/xhtml\">A human-readable rendering of the ClaimResponse</div>"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.total[0].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.total[0].amount.value, 135.57)
        self.assertEqual(force_bytes(inst.total[0].category.coding[0].code), force_bytes("submitted"))
        self.assertEqual(force_bytes(inst.total[1].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.total[1].amount.value, 90.47)
        self.assertEqual(force_bytes(inst.total[1].category.coding[0].code), force_bytes("benefit"))
        self.assertEqual(force_bytes(inst.type.coding[0].code), force_bytes("oral"))
        self.assertEqual(force_bytes(inst.type.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/claim-type"))
        self.assertEqual(force_bytes(inst.use), force_bytes("claim"))
    
    def testClaimResponse5(self):
        inst = self.instantiate_from("claimresponse-example-unsolicited-preauth.json")
        self.assertIsNotNone(inst, "Must have instantiated a ClaimResponse instance")
        self.implClaimResponse5(inst)
        
        js = inst.as_json()
        self.assertEqual("ClaimResponse", js["resourceType"])
        inst2 = claimresponse.ClaimResponse(js)
        self.implClaimResponse5(inst2)
    
    def implClaimResponse5(self, inst):
        self.assertEqual(force_bytes(inst.addItem[0].adjudication[0].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.addItem[0].adjudication[0].amount.value, 250.0)
        self.assertEqual(force_bytes(inst.addItem[0].adjudication[0].category.coding[0].code), force_bytes("eligible"))
        self.assertEqual(force_bytes(inst.addItem[0].adjudication[1].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.addItem[0].adjudication[1].amount.value, 10.0)
        self.assertEqual(force_bytes(inst.addItem[0].adjudication[1].category.coding[0].code), force_bytes("copay"))
        self.assertEqual(force_bytes(inst.addItem[0].adjudication[2].category.coding[0].code), force_bytes("eligpercent"))
        self.assertEqual(inst.addItem[0].adjudication[2].value, 100.0)
        self.assertEqual(force_bytes(inst.addItem[0].adjudication[3].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.addItem[0].adjudication[3].amount.value, 240.0)
        self.assertEqual(force_bytes(inst.addItem[0].adjudication[3].category.coding[0].code), force_bytes("benefit"))
        self.assertEqual(inst.addItem[0].itemSequence[0], 1)
        self.assertEqual(force_bytes(inst.addItem[0].modifier[0].coding[0].code), force_bytes("x"))
        self.assertEqual(force_bytes(inst.addItem[0].modifier[0].coding[0].display), force_bytes("None"))
        self.assertEqual(force_bytes(inst.addItem[0].modifier[0].coding[0].system), force_bytes("http://example.org/fhir/modifiers"))
        self.assertEqual(force_bytes(inst.addItem[0].net.currency), force_bytes("USD"))
        self.assertEqual(inst.addItem[0].net.value, 250.0)
        self.assertEqual(inst.addItem[0].noteNumber[0], 101)
        self.assertEqual(force_bytes(inst.addItem[0].productOrService.coding[0].code), force_bytes("1101"))
        self.assertEqual(force_bytes(inst.addItem[0].productOrService.coding[0].system), force_bytes("http://example.org/fhir/oralservicecodes"))
        self.assertEqual(force_bytes(inst.addItem[1].adjudication[0].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.addItem[1].adjudication[0].amount.value, 800.0)
        self.assertEqual(force_bytes(inst.addItem[1].adjudication[0].category.coding[0].code), force_bytes("eligible"))
        self.assertEqual(force_bytes(inst.addItem[1].adjudication[1].category.coding[0].code), force_bytes("eligpercent"))
        self.assertEqual(inst.addItem[1].adjudication[1].value, 100.0)
        self.assertEqual(force_bytes(inst.addItem[1].adjudication[2].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.addItem[1].adjudication[2].amount.value, 800.0)
        self.assertEqual(force_bytes(inst.addItem[1].adjudication[2].category.coding[0].code), force_bytes("benefit"))
        self.assertEqual(inst.addItem[1].itemSequence[0], 1)
        self.assertEqual(force_bytes(inst.addItem[1].net.currency), force_bytes("USD"))
        self.assertEqual(inst.addItem[1].net.value, 800.0)
        self.assertEqual(force_bytes(inst.addItem[1].productOrService.coding[0].code), force_bytes("2101"))
        self.assertEqual(force_bytes(inst.addItem[1].productOrService.coding[0].display), force_bytes("Radiograph, series (12)"))
        self.assertEqual(force_bytes(inst.addItem[1].productOrService.coding[0].system), force_bytes("http://example.org/fhir/oralservicecodes"))
        self.assertEqual(inst.created.date, FHIRDate("2014-08-16").date)
        self.assertEqual(inst.created.as_json(), "2014-08-16")
        self.assertEqual(force_bytes(inst.disposition), force_bytes("The enclosed services are authorized for your provision within 30 days of this notice."))
        self.assertEqual(force_bytes(inst.id), force_bytes("UR3503"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("http://www.SocialBenefitsInc.com/fhir/ClaimResponse"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("UR3503"))
        self.assertTrue(inst.insurance[0].focal)
        self.assertEqual(inst.insurance[0].sequence, 1)
        self.assertEqual(force_bytes(inst.meta.tag[0].code), force_bytes("HTEST"))
        self.assertEqual(force_bytes(inst.meta.tag[0].display), force_bytes("test health data"))
        self.assertEqual(force_bytes(inst.meta.tag[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/v3-ActReason"))
        self.assertEqual(force_bytes(inst.outcome), force_bytes("complete"))
        self.assertEqual(force_bytes(inst.payeeType.coding[0].code), force_bytes("provider"))
        self.assertEqual(force_bytes(inst.payeeType.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/payeetype"))
        self.assertEqual(force_bytes(inst.preAuthRef), force_bytes("18SS12345"))
        self.assertEqual(force_bytes(inst.processNote[0].language.coding[0].code), force_bytes("en-CA"))
        self.assertEqual(force_bytes(inst.processNote[0].language.coding[0].system), force_bytes("urn:ietf:bcp:47"))
        self.assertEqual(inst.processNote[0].number, 101)
        self.assertEqual(force_bytes(inst.processNote[0].text), force_bytes("Please submit a Pre-Authorization request if a more extensive examination or urgent services are required."))
        self.assertEqual(force_bytes(inst.processNote[0].type), force_bytes("print"))
        self.assertEqual(force_bytes(inst.status), force_bytes("active"))
        self.assertEqual(force_bytes(inst.text.div), force_bytes("<div xmlns=\"http://www.w3.org/1999/xhtml\">A sample unsolicited pre-authorization response which authorizes basic dental services to be performed for a patient.</div>"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.total[0].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.total[0].amount.value, 1050.0)
        self.assertEqual(force_bytes(inst.total[0].category.coding[0].code), force_bytes("submitted"))
        self.assertEqual(force_bytes(inst.total[1].amount.currency), force_bytes("USD"))
        self.assertEqual(inst.total[1].amount.value, 1040.0)
        self.assertEqual(force_bytes(inst.total[1].category.coding[0].code), force_bytes("benefit"))
        self.assertEqual(force_bytes(inst.type.coding[0].code), force_bytes("oral"))
        self.assertEqual(force_bytes(inst.type.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/claim-type"))
        self.assertEqual(force_bytes(inst.use), force_bytes("preauthorization"))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.0-a53ec6ee1b (http://hl7.org/fhir/StructureDefinition/SupplyRequest) on 2019-05-13.
#  2019, SMART Health IT.


from . import domainresource

class SupplyRequest(domainresource.DomainResource):
    """ Request for a medication, substance or device.

    A record of a request for a medication, substance or device used in the
    healthcare setting.
    """

    resource_type = "SupplyRequest"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.authoredOn = None
        """ When the request was made.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.category = None
        """ The kind of supply (central, non-stock, etc.).
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.deliverFrom = None
        """ The origin of the supply.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.deliverTo = None
        """ The destination of the supply.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.identifier = None
        """ Business Identifier for SupplyRequest.
        List of `Identifier` items (represented as `dict` in JSON). """

        self.itemCodeableConcept = None
        """ Medication, Substance, or Device requested to be supplied.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.itemReference = None
        """ Medication, Substance, or Device requested to be supplied.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.occurrenceDateTime = None
        """ When the request should be fulfilled.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.occurrencePeriod = None
        """ When the request should be fulfilled.
        Type `Period` (represented as `dict` in JSON). """

        self.occurrenceTiming = None
        """ When the request should be fulfilled.
        Type `Timing` (represented as `dict` in JSON). """

        self.parameter = None
        """ Ordered item details.
        List of `SupplyRequestParameter` items (represented as `dict` in JSON). """

        self.priority = None
        """ routine | urgent | asap | stat.
        Type `str`. """

        self.quantity = None
        """ The requested amount of the item indicated.
        Type `Quantity` (represented as `dict` in JSON). """

        self.reasonCode = None
        """ The reason why the supply item was requested.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.reasonReference = None
        """ The reason why the supply item was requested.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.requester = None
        """ Individual making the request.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.status = None
        """ draft | active | suspended +.
        Type `str`. """

        self.supplier = None
        """ Who is intended to fulfill the request.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        super(SupplyRequest, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(SupplyRequest, self).elementProperties()
        js.extend([
            ("authoredOn", "authoredOn", fhirdate.FHIRDate, "dateTime", False, None, False),
            ("category", "category", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("deliverFrom", "deliverFrom", fhirreference.FHIRReference, "Reference", False, None, False),
            ("deliverTo", "deliverTo", fhirreference.FHIRReference, "Reference", False, None, False),
            ("identifier", "identifier", identifier.Identifier, "Identifier", True, None, False),
            ("itemCodeableConcept", "itemCodeableConcept", codeableconcept.CodeableConcept, "CodeableConcept", False, "item", True),
            ("itemReference", "itemReference", fhirreference.FHIRReference, "Reference", False, "item", True),
            ("occurrenceDateTime", "occurrenceDateTime", fhirdate.FHIRDate, "dateTime", False, "occurrence", False),
            ("occurrencePeriod", "occurrencePeriod", period.Period, "Period", False, "occurrence", False),
            ("occurrenceTiming", "occurrenceTiming", timing.Timing, "Timing", False, "occurrence", False),
            ("parameter", "parameter", SupplyRequestParameter, "SupplyRequestParameter", True, None, False),
            ("priority", "priority", str, "code", False, None, False),
            ("quantity", "quantity", quantity.Quantity, "Quantity", False, None, True),
            ("reasonCode", "reasonCode", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("reasonReference", "reasonReference", fhirreference.FHIRReference, "Reference", True, None, False),
            ("requester", "requester", fhirreference.FHIRReference, "Reference", False, None, False),
            ("status", "status", str, "code", False, None, False),
            ("supplier", "supplier", fhirreference.FHIRReference, "Reference", True, None, False),
        ])
        return js


from . import backboneelement

class SupplyRequestParameter(backboneelement.BackboneElement):
    """ Ordered item details.

    Specific parameters for the ordered item.  For example, the size of the
    indicated item.
    """

    resource_type = "SupplyRequestParameter"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.code = None
        """ Item detail.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.valueBoolean = None
        """ Value of detail.
        Type `bool`. """

        self.valueCodeableConcept = None
        """ Value of detail.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.valueQuantity = None
        """ Value of detail.
        Type `Quantity` (represented as `dict` in JSON). """

        self.valueRange = None
        """ Value of detail.
        Type `Range` (represented as `dict` in JSON). """

        super(SupplyRequestParameter, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(SupplyRequestParameter, self).elementProperties()
        js.extend([
            ("code", "code", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("valueBoolean", "valueBoolean", bool, "boolean", False, "value", False),
            ("valueCodeableConcept", "valueCodeableConcept", codeableconcept.CodeableConcept, "CodeableConcept", False, "value", False),
            ("valueQuantity", "valueQuantity", quantity.Quantity, "Quantity", False, "value", False),
            ("valueRange", "valueRange", range.Range, "Range", False, "value", False),
        ])
        return js


import sys
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import fhirdate
except ImportError:
    fhirdate = sys.modules[__package__ + '.fhirdate']
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']
try:
    from . import identifier
except ImportError:
    identifier = sys.modules[__package__ + '.identifier']
try:
    from . import period
except ImportError:
    period = sys.modules[__package__ + '.period']
try:
    from . import quantity
except ImportError:
    quantity = sys.modules[__package__ + '.quantity']
try:
    from . import range
except ImportError:
    range = sys.modules[__package__ + '.range']
try:
    from . import timing
except ImportError:
    timing = sys.modules[__package__ + '.timing']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 3.0.1.11917 (http://hl7.org/fhir/StructureDefinition/Duration) on 2019-05-13.
#  2019, SMART Health IT.


from . import quantity

class Duration(quantity.Quantity):
    """ A length of time.
    """

    resource_type = "Duration"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        super(Duration, self).__init__(jsondict=jsondict, strict=strict)





#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.0-a53ec6ee1b (http://hl7.org/fhir/StructureDefinition/Schedule) on 2019-05-13.
#  2019, SMART Health IT.


from . import domainresource

class Schedule(domainresource.DomainResource):
    """ A container for slots of time that may be available for booking
    appointments.
    """

    resource_type = "Schedule"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.active = None
        """ Whether this schedule is in active use.
        Type `bool`. """

        self.actor = None
        """ Resource(s) that availability information is being provided for.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.comment = None
        """ Comments on availability.
        Type `str`. """

        self.identifier = None
        """ External Ids for this item.
        List of `Identifier` items (represented as `dict` in JSON). """

        self.planningHorizon = None
        """ Period of time covered by schedule.
        Type `Period` (represented as `dict` in JSON). """

        self.serviceCategory = None
        """ High-level category.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.serviceType = None
        """ Specific service.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.specialty = None
        """ Type of specialty needed.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        super(Schedule, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(Schedule, self).elementProperties()
        js.extend([
            ("active", "active", bool, "boolean", False, None, False),
            ("actor", "actor", fhirreference.FHIRReference, "Reference", True, None, True),
            ("comment", "comment", str, "string", False, None, False),
            ("identifier", "identifier", identifier.Identifier, "Identifier", True, None, False),
            ("planningHorizon", "planningHorizon", period.Period, "Period", False, None, False),
            ("serviceCategory", "serviceCategory", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("serviceType", "serviceType", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("specialty", "specialty", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
        ])
        return js


import sys
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']
try:
    from . import identifier
except ImportError:
    identifier = sys.modules[__package__ + '.identifier']
try:
    from . import period
except ImportError:
    period = sys.modules[__package__ + '.period']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 3.0.1.11917 on 2019-05-13.
#  2019, SMART Health IT.

import os
import pytest
import io
import unittest
import json

from .fixtures import force_bytes
from .. import supplyrequest
from ..fhirdate import FHIRDate


@pytest.mark.usefixtures("base_settings")
class SupplyRequestTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("SupplyRequest", js["resourceType"])
        return supplyrequest.SupplyRequest(js)
    
    def testSupplyRequest1(self):
        inst = self.instantiate_from("supplyrequest-example-simpleorder.json")
        self.assertIsNotNone(inst, "Must have instantiated a SupplyRequest instance")
        self.implSupplyRequest1(inst)
        
        js = inst.as_json()
        self.assertEqual("SupplyRequest", js["resourceType"])
        inst2 = supplyrequest.SupplyRequest(js)
        self.implSupplyRequest1(inst2)
    
    def implSupplyRequest1(self, inst):
        self.assertEqual(inst.authoredOn.date, FHIRDate("2016-12-31").date)
        self.assertEqual(inst.authoredOn.as_json(), "2016-12-31")
        self.assertEqual(force_bytes(inst.category.coding[0].code), force_bytes("central"))
        self.assertEqual(force_bytes(inst.category.coding[0].display), force_bytes("Central Stock Resupply"))
        self.assertEqual(force_bytes(inst.id), force_bytes("simpleorder"))
        self.assertEqual(force_bytes(inst.identifier.value), force_bytes("Order10284"))
        self.assertEqual(inst.occurrenceDateTime.date, FHIRDate("2016-12-31").date)
        self.assertEqual(inst.occurrenceDateTime.as_json(), "2016-12-31")
        self.assertEqual(force_bytes(inst.orderedItem.itemCodeableConcept.coding[0].code), force_bytes("BlueTubes"))
        self.assertEqual(force_bytes(inst.orderedItem.itemCodeableConcept.coding[0].display), force_bytes("Blood collect tubes blue cap"))
        self.assertEqual(inst.orderedItem.quantity.value, 10)
        self.assertEqual(force_bytes(inst.priority), force_bytes("asap"))
        self.assertEqual(force_bytes(inst.reasonCodeableConcept.coding[0].code), force_bytes("stock_low"))
        self.assertEqual(force_bytes(inst.reasonCodeableConcept.coding[0].display), force_bytes("Refill due to low stock"))
        self.assertEqual(force_bytes(inst.status), force_bytes("active"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 3.0.1.11917 (http://hl7.org/fhir/StructureDefinition/DeviceRequest) on 2019-05-13.
#  2019, SMART Health IT.


from . import domainresource

class DeviceRequest(domainresource.DomainResource):
    """ Medical device request.

    Represents a request for a patient to employ a medical device. The device
    may be an implantable device, or an external assistive device, such as a
    walker.
    """

    resource_type = "DeviceRequest"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.authoredOn = None
        """ When recorded.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.basedOn = None
        """ What request fulfills.
        List of `FHIRReference` items referencing `Resource` (represented as `dict` in JSON). """

        self.codeCodeableConcept = None
        """ Device requested.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.codeReference = None
        """ Device requested.
        Type `FHIRReference` referencing `Device` (represented as `dict` in JSON). """

        self.context = None
        """ Encounter or Episode motivating request.
        Type `FHIRReference` referencing `Encounter, EpisodeOfCare` (represented as `dict` in JSON). """

        self.definition = None
        """ Protocol or definition.
        List of `FHIRReference` items referencing `ActivityDefinition, PlanDefinition` (represented as `dict` in JSON). """

        self.groupIdentifier = None
        """ Identifier of composite request.
        Type `Identifier` (represented as `dict` in JSON). """

        self.identifier = None
        """ External Request identifier.
        List of `Identifier` items (represented as `dict` in JSON). """

        self.intent = None
        """ proposal | plan | original-order | encoded | reflex-order.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.note = None
        """ Notes or comments.
        List of `Annotation` items (represented as `dict` in JSON). """

        self.occurrenceDateTime = None
        """ Desired time or schedule for use.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.occurrencePeriod = None
        """ Desired time or schedule for use.
        Type `Period` (represented as `dict` in JSON). """

        self.occurrenceTiming = None
        """ Desired time or schedule for use.
        Type `Timing` (represented as `dict` in JSON). """

        self.performer = None
        """ Requested Filler.
        Type `FHIRReference` referencing `Practitioner, Organization, Patient, Device, RelatedPerson, HealthcareService` (represented as `dict` in JSON). """

        self.performerType = None
        """ Fille role.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.priorRequest = None
        """ What request replaces.
        List of `FHIRReference` items referencing `Resource` (represented as `dict` in JSON). """

        self.priority = None
        """ Indicates how quickly the {{title}} should be addressed with
        respect to other requests.
        Type `str`. """

        self.reasonCode = None
        """ Coded Reason for request.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.reasonReference = None
        """ Linked Reason for request.
        List of `FHIRReference` items referencing `Resource` (represented as `dict` in JSON). """

        self.relevantHistory = None
        """ Request provenance.
        List of `FHIRReference` items referencing `Provenance` (represented as `dict` in JSON). """

        self.requester = None
        """ Who/what is requesting diagnostics.
        Type `DeviceRequestRequester` (represented as `dict` in JSON). """

        self.status = None
        """ draft | active | suspended | completed | entered-in-error |
        cancelled.
        Type `str`. """

        self.subject = None
        """ Focus of request.
        Type `FHIRReference` referencing `Patient, Group, Location, Device` (represented as `dict` in JSON). """

        self.supportingInfo = None
        """ Additional clinical information.
        List of `FHIRReference` items referencing `Resource` (represented as `dict` in JSON). """

        super(DeviceRequest, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(DeviceRequest, self).elementProperties()
        js.extend([
            ("authoredOn", "authoredOn", fhirdate.FHIRDate, "dateTime", False, None, False),
            ("basedOn", "basedOn", fhirreference.FHIRReference, "Reference", True, None, False),
            ("codeCodeableConcept", "codeCodeableConcept", codeableconcept.CodeableConcept, "CodeableConcept", False, "code", True),
            ("codeReference", "codeReference", fhirreference.FHIRReference, "Reference", False, "code", True),
            ("context", "context", fhirreference.FHIRReference, "Reference", False, None, False),
            ("definition", "definition", fhirreference.FHIRReference, "Reference", True, None, False),
            ("groupIdentifier", "groupIdentifier", identifier.Identifier, "Identifier", False, None, False),
            ("identifier", "identifier", identifier.Identifier, "Identifier", True, None, False),
            ("intent", "intent", codeableconcept.CodeableConcept, "CodeableConcept", False, None, True),
            ("note", "note", annotation.Annotation, "Annotation", True, None, False),
            ("occurrenceDateTime", "occurrenceDateTime", fhirdate.FHIRDate, "dateTime", False, "occurrence", False),
            ("occurrencePeriod", "occurrencePeriod", period.Period, "Period", False, "occurrence", False),
            ("occurrenceTiming", "occurrenceTiming", timing.Timing, "Timing", False, "occurrence", False),
            ("performer", "performer", fhirreference.FHIRReference, "Reference", False, None, False),
            ("performerType", "performerType", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("priorRequest", "priorRequest", fhirreference.FHIRReference, "Reference", True, None, False),
            ("priority", "priority", str, "code", False, None, False),
            ("reasonCode", "reasonCode", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("reasonReference", "reasonReference", fhirreference.FHIRReference, "Reference", True, None, False),
            ("relevantHistory", "relevantHistory", fhirreference.FHIRReference, "Reference", True, None, False),
            ("requester", "requester", DeviceRequestRequester, "DeviceRequestRequester", False, None, False),
            ("status", "status", str, "code", False, None, False),
            ("subject", "subject", fhirreference.FHIRReference, "Reference", False, None, True),
            ("supportingInfo", "supportingInfo", fhirreference.FHIRReference, "Reference", True, None, False),
        ])
        return js


from . import backboneelement

class DeviceRequestRequester(backboneelement.BackboneElement):
    """ Who/what is requesting diagnostics.

    The individual who initiated the request and has responsibility for its
    activation.
    """

    resource_type = "DeviceRequestRequester"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.agent = None
        """ Individual making the request.
        Type `FHIRReference` referencing `Device, Practitioner, Organization` (represented as `dict` in JSON). """

        self.onBehalfOf = None
        """ Organization agent is acting for.
        Type `FHIRReference` referencing `Organization` (represented as `dict` in JSON). """

        super(DeviceRequestRequester, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(DeviceRequestRequester, self).elementProperties()
        js.extend([
            ("agent", "agent", fhirreference.FHIRReference, "Reference", False, None, True),
            ("onBehalfOf", "onBehalfOf", fhirreference.FHIRReference, "Reference", False, None, False),
        ])
        return js


import sys
try:
    from . import annotation
except ImportError:
    annotation = sys.modules[__package__ + '.annotation']
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import fhirdate
except ImportError:
    fhirdate = sys.modules[__package__ + '.fhirdate']
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']
try:
    from . import identifier
except ImportError:
    identifier = sys.modules[__package__ + '.identifier']
try:
    from . import period
except ImportError:
    period = sys.modules[__package__ + '.period']
try:
    from . import timing
except ImportError:
    timing = sys.modules[__package__ + '.timing']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.0-a53ec6ee1b on 2019-05-13.
#  2019, SMART Health IT.

import os
import pytest
import io
import unittest
import json

from .fixtures import force_bytes
from .. import contract
from ..fhirdate import FHIRDate


@pytest.mark.usefixtures("base_settings")
class ContractTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("Contract", js["resourceType"])
        return contract.Contract(js)
    
    def testContract1(self):
        inst = self.instantiate_from("pcd-example-notOrg.json")
        self.assertIsNotNone(inst, "Must have instantiated a Contract instance")
        self.implContract1(inst)
        
        js = inst.as_json()
        self.assertEqual("Contract", js["resourceType"])
        inst2 = contract.Contract(js)
        self.implContract1(inst2)
    
    def implContract1(self, inst):
        self.assertEqual(force_bytes(inst.friendly[0].contentAttachment.title), force_bytes("The terms of the consent in friendly consumer speak."))
        self.assertEqual(force_bytes(inst.id), force_bytes("pcd-example-notOrg"))
        self.assertEqual(inst.issued.date, FHIRDate("2015-11-18").date)
        self.assertEqual(inst.issued.as_json(), "2015-11-18")
        self.assertEqual(force_bytes(inst.legal[0].contentAttachment.title), force_bytes("The terms of the consent in lawyer speak."))
        self.assertEqual(force_bytes(inst.meta.tag[0].code), force_bytes("HTEST"))
        self.assertEqual(force_bytes(inst.meta.tag[0].display), force_bytes("test health data"))
        self.assertEqual(force_bytes(inst.meta.tag[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/v3-ActReason"))
        self.assertEqual(force_bytes(inst.subType[0].coding[0].code), force_bytes("Opt-In"))
        self.assertEqual(force_bytes(inst.subType[0].coding[0].display), force_bytes("Default Authorization with exceptions."))
        self.assertEqual(force_bytes(inst.subType[0].coding[0].system), force_bytes("http://www.infoway-inforoute.ca.org/Consent-subtype-codes"))
        self.assertEqual(force_bytes(inst.term[0].offer.text), force_bytes("Withhold this order and any results or related objects from any provider."))
        self.assertEqual(force_bytes(inst.term[0].type.coding[0].code), force_bytes("withhold-from"))
        self.assertEqual(force_bytes(inst.term[0].type.coding[0].display), force_bytes("Withhold all data from specified actor entity."))
        self.assertEqual(force_bytes(inst.term[0].type.coding[0].system), force_bytes("http://example.org/fhir/consent-term-type-codes"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.type.coding[0].code), force_bytes("57016-8"))
        self.assertEqual(force_bytes(inst.type.coding[0].system), force_bytes("http://loinc.org"))
    
    def testContract2(self):
        inst = self.instantiate_from("pcd-example-notLabs.json")
        self.assertIsNotNone(inst, "Must have instantiated a Contract instance")
        self.implContract2(inst)
        
        js = inst.as_json()
        self.assertEqual("Contract", js["resourceType"])
        inst2 = contract.Contract(js)
        self.implContract2(inst2)
    
    def implContract2(self, inst):
        self.assertEqual(force_bytes(inst.friendly[0].contentAttachment.title), force_bytes("The terms of the consent in friendly consumer speak."))
        self.assertEqual(force_bytes(inst.id), force_bytes("pcd-example-notLabs"))
        self.assertEqual(inst.issued.date, FHIRDate("2014-08-17").date)
        self.assertEqual(inst.issued.as_json(), "2014-08-17")
        self.assertEqual(force_bytes(inst.legal[0].contentAttachment.title), force_bytes("The terms of the consent in lawyer speak."))
        self.assertEqual(force_bytes(inst.meta.tag[0].code), force_bytes("HTEST"))
        self.assertEqual(force_bytes(inst.meta.tag[0].display), force_bytes("test health data"))
        self.assertEqual(force_bytes(inst.meta.tag[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/v3-ActReason"))
        self.assertEqual(force_bytes(inst.subType[0].coding[0].code), force_bytes("Opt-In"))
        self.assertEqual(force_bytes(inst.subType[0].coding[0].display), force_bytes("Default Authorization with exceptions."))
        self.assertEqual(force_bytes(inst.subType[0].coding[0].system), force_bytes("http://www.infoway-inforoute.ca.org/Consent-subtype-codes"))
        self.assertEqual(force_bytes(inst.term[0].group[0].offer.text), force_bytes("Withhold orders from any provider."))
        self.assertEqual(force_bytes(inst.term[0].group[0].subType.coding[0].code), force_bytes("ServiceRequest"))
        self.assertEqual(force_bytes(inst.term[0].group[0].subType.coding[0].system), force_bytes("http://hl7.org/fhir/resource-types"))
        self.assertEqual(force_bytes(inst.term[0].group[0].type.coding[0].code), force_bytes("withhold-object-type"))
        self.assertEqual(force_bytes(inst.term[0].group[0].type.coding[0].system), force_bytes("http://example.org/fhir/consent-term-type-codes"))
        self.assertEqual(force_bytes(inst.term[0].group[1].offer.text), force_bytes("Withhold order results from any provider."))
        self.assertEqual(force_bytes(inst.term[0].group[1].subType.coding[0].code), force_bytes("DiagnosticReport"))
        self.assertEqual(force_bytes(inst.term[0].group[1].subType.coding[0].system), force_bytes("http://hl7.org/fhir/resource-types"))
        self.assertEqual(force_bytes(inst.term[0].group[1].type.coding[0].code), force_bytes("withhold-object-type"))
        self.assertEqual(force_bytes(inst.term[0].group[1].type.coding[0].system), force_bytes("http://example.org/fhir/consent-term-type-codes"))
        self.assertEqual(force_bytes(inst.term[0].offer.text), force_bytes("sample"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.type.coding[0].code), force_bytes("57016-8"))
        self.assertEqual(force_bytes(inst.type.coding[0].system), force_bytes("http://loinc.org"))
    
    def testContract3(self):
        inst = self.instantiate_from("contract-example-42cfr-part2.json")
        self.assertIsNotNone(inst, "Must have instantiated a Contract instance")
        self.implContract3(inst)
        
        js = inst.as_json()
        self.assertEqual("Contract", js["resourceType"])
        inst2 = contract.Contract(js)
        self.implContract3(inst2)
    
    def implContract3(self, inst):
        self.assertEqual(inst.applies.start.date, FHIRDate("2013-11-01T21:18:27-04:00").date)
        self.assertEqual(inst.applies.start.as_json(), "2013-11-01T21:18:27-04:00")
        self.assertEqual(force_bytes(inst.contentDerivative.coding[0].code), force_bytes("registration"))
        self.assertEqual(force_bytes(inst.contentDerivative.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/contract-content-derivative"))
        self.assertEqual(force_bytes(inst.id), force_bytes("C-2121"))
        self.assertEqual(inst.issued.date, FHIRDate("2013-11-01T21:18:27-04:00").date)
        self.assertEqual(inst.issued.as_json(), "2013-11-01T21:18:27-04:00")
        self.assertEqual(force_bytes(inst.legal[0].contentAttachment.contentType), force_bytes("application/pdf"))
        self.assertEqual(force_bytes(inst.legal[0].contentAttachment.language), force_bytes("en-US"))
        self.assertEqual(force_bytes(inst.legal[0].contentAttachment.title), force_bytes("MDHHS-5515 Consent To Share Your Health Information"))
        self.assertEqual(force_bytes(inst.legal[0].contentAttachment.url), force_bytes("http://org.mihin.ecms/ConsentDirective-2121"))
        self.assertEqual(inst.meta.lastUpdated.date, FHIRDate("2016-07-19T18:18:42.108-04:00").date)
        self.assertEqual(inst.meta.lastUpdated.as_json(), "2016-07-19T18:18:42.108-04:00")
        self.assertEqual(force_bytes(inst.meta.tag[0].code), force_bytes("HTEST"))
        self.assertEqual(force_bytes(inst.meta.tag[0].display), force_bytes("test health data"))
        self.assertEqual(force_bytes(inst.meta.tag[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/v3-ActReason"))
        self.assertEqual(force_bytes(inst.meta.versionId), force_bytes("1"))
        self.assertEqual(force_bytes(inst.signer[0].signature[0].type[0].code), force_bytes("1.2.840.10065.1.12.1.1"))
        self.assertEqual(force_bytes(inst.signer[0].signature[0].type[0].system), force_bytes("urn:iso-astm:E1762-95:2013"))
        self.assertEqual(inst.signer[0].signature[0].when.date, FHIRDate("2017-02-08T10:57:34+01:00").date)
        self.assertEqual(inst.signer[0].signature[0].when.as_json(), "2017-02-08T10:57:34+01:00")
        self.assertEqual(force_bytes(inst.signer[0].type.code), force_bytes("SELF"))
        self.assertEqual(force_bytes(inst.signer[0].type.system), force_bytes("http://mdhhs.org/fhir/consent-signer-type"))
        self.assertEqual(force_bytes(inst.status), force_bytes("executed"))
        self.assertEqual(force_bytes(inst.subType[0].coding[0].code), force_bytes("hcd"))
        self.assertEqual(force_bytes(inst.subType[0].coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/consentcategorycodes"))
        self.assertEqual(force_bytes(inst.term[0].action[0].intent.coding[0].code), force_bytes("HPRGRP"))
        self.assertEqual(force_bytes(inst.term[0].action[0].intent.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/v3-ActReason"))
        self.assertEqual(force_bytes(inst.term[0].action[0].status.text), force_bytes("Sample"))
        self.assertEqual(force_bytes(inst.term[0].action[0].subject[0].role.coding[0].code), force_bytes("IR"))
        self.assertEqual(force_bytes(inst.term[0].action[0].subject[0].role.coding[0].display), force_bytes("Recipient"))
        self.assertEqual(force_bytes(inst.term[0].action[0].subject[0].role.coding[0].system), force_bytes("http://mdhhs.org/fhir/consent-actor-type"))
        self.assertEqual(force_bytes(inst.term[0].action[0].subject[0].role.text), force_bytes("Recipient of restricted health information"))
        self.assertEqual(force_bytes(inst.term[0].action[0].subject[1].role.coding[0].code), force_bytes("IS"))
        self.assertEqual(force_bytes(inst.term[0].action[0].subject[1].role.coding[0].display), force_bytes("Sender"))
        self.assertEqual(force_bytes(inst.term[0].action[0].subject[1].role.coding[0].system), force_bytes("http://mdhhs.org/fhir/consent-actor-type"))
        self.assertEqual(force_bytes(inst.term[0].action[0].subject[1].role.text), force_bytes("Sender of restricted health information"))
        self.assertEqual(force_bytes(inst.term[0].action[0].type.coding[0].code), force_bytes("action-a"))
        self.assertEqual(force_bytes(inst.term[0].action[0].type.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/contractaction"))
        self.assertEqual(inst.term[0].asset[0].period[0].end.date, FHIRDate("2019-11-01T21:18:27-04:00").date)
        self.assertEqual(inst.term[0].asset[0].period[0].end.as_json(), "2019-11-01T21:18:27-04:00")
        self.assertEqual(inst.term[0].asset[0].period[0].start.date, FHIRDate("2013-11-01T21:18:27-04:00").date)
        self.assertEqual(inst.term[0].asset[0].period[0].start.as_json(), "2013-11-01T21:18:27-04:00")
        self.assertEqual(force_bytes(inst.term[0].offer.decision.coding[0].code), force_bytes("OPTIN"))
        self.assertEqual(force_bytes(inst.term[0].offer.decision.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/v3-ActCode"))
        self.assertEqual(force_bytes(inst.term[0].offer.text), force_bytes("Can't refuse"))
        self.assertEqual(force_bytes(inst.term[0].offer.type.coding[0].code), force_bytes("statutory"))
        self.assertEqual(force_bytes(inst.term[0].offer.type.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/contracttermtypecodes"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.type.coding[0].code), force_bytes("OPTIN"))
        self.assertEqual(force_bytes(inst.type.coding[0].system), force_bytes("http://mdhhs.org/fhir/consentdirective-type"))
        self.assertEqual(force_bytes(inst.type.text), force_bytes("Opt-in consent directive"))
    
    def testContract4(self):
        inst = self.instantiate_from("contract-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a Contract instance")
        self.implContract4(inst)
        
        js = inst.as_json()
        self.assertEqual("Contract", js["resourceType"])
        inst2 = contract.Contract(js)
        self.implContract4(inst2)
    
    def implContract4(self, inst):
        self.assertEqual(force_bytes(inst.id), force_bytes("C-123"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("http://happyvalley.com/contract"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("12347"))
        self.assertEqual(force_bytes(inst.legallyBindingAttachment.contentType), force_bytes("application/pdf"))
        self.assertEqual(force_bytes(inst.legallyBindingAttachment.url), force_bytes("http://www.aws3.com/storage/doc.pdf"))
        self.assertEqual(force_bytes(inst.meta.tag[0].code), force_bytes("HTEST"))
        self.assertEqual(force_bytes(inst.meta.tag[0].display), force_bytes("test health data"))
        self.assertEqual(force_bytes(inst.meta.tag[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/v3-ActReason"))
        self.assertEqual(force_bytes(inst.rule[0].contentAttachment.contentType), force_bytes("application/txt"))
        self.assertEqual(force_bytes(inst.rule[0].contentAttachment.url), force_bytes("http://www.rfc-editor.org/bcp/bcp13.txt"))
        self.assertEqual(inst.term[0].asset[0].period[0].start.date, FHIRDate("2017-06-01").date)
        self.assertEqual(inst.term[0].asset[0].period[0].start.as_json(), "2017-06-01")
        self.assertEqual(force_bytes(inst.term[0].asset[0].subtype[0].text), force_bytes("sample"))
        self.assertEqual(force_bytes(inst.term[0].asset[0].type[0].coding[0].code), force_bytes("RicardianContract"))
        self.assertEqual(force_bytes(inst.term[0].asset[0].type[0].coding[0].system), force_bytes("urn:ietf:rfc:3986"))
        self.assertEqual(inst.term[0].asset[0].valuedItem[0].effectiveTime.date, FHIRDate("1995").date)
        self.assertEqual(inst.term[0].asset[0].valuedItem[0].effectiveTime.as_json(), "1995")
        self.assertEqual(force_bytes(inst.term[0].asset[0].valuedItem[0].entityCodeableConcept.text), force_bytes("Ford Bobcat"))
        self.assertEqual(inst.term[0].asset[0].valuedItem[0].factor, 1.0)
        self.assertEqual(force_bytes(inst.term[0].asset[0].valuedItem[0].identifier.system), force_bytes("http://somewhere.motor-vehicle.com/vin"))
        self.assertEqual(force_bytes(inst.term[0].asset[0].valuedItem[0].identifier.value), force_bytes("XXSVT34-7665t952236"))
        self.assertEqual(force_bytes(inst.term[0].asset[0].valuedItem[0].net.currency), force_bytes("CAD"))
        self.assertEqual(inst.term[0].asset[0].valuedItem[0].net.value, 200.0)
        self.assertEqual(inst.term[0].asset[0].valuedItem[0].points, 1.0)
        self.assertEqual(inst.term[0].asset[0].valuedItem[0].quantity.value, 1)
        self.assertEqual(force_bytes(inst.term[0].asset[0].valuedItem[0].unitPrice.currency), force_bytes("CAD"))
        self.assertEqual(inst.term[0].asset[0].valuedItem[0].unitPrice.value, 200.0)
        self.assertEqual(force_bytes(inst.term[0].offer.text), force_bytes("Can't refuse"))
        self.assertEqual(force_bytes(inst.text.div), force_bytes("<div xmlns=\"http://www.w3.org/1999/xhtml\">A human-readable rendering of the contract</div>"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
    
    def testContract5(self):
        inst = self.instantiate_from("contract-example-ins-policy.json")
        self.assertIsNotNone(inst, "Must have instantiated a Contract instance")
        self.implContract5(inst)
        
        js = inst.as_json()
        self.assertEqual("Contract", js["resourceType"])
        inst2 = contract.Contract(js)
        self.implContract5(inst2)
    
    def implContract5(self, inst):
        self.assertEqual(inst.applies.start.date, FHIRDate("2017-01-01").date)
        self.assertEqual(inst.applies.start.as_json(), "2017-01-01")
        self.assertEqual(force_bytes(inst.id), force_bytes("INS-101"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("http://xyz-insurance.com/forms"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("YCSCWLN(01-2017)"))
        self.assertEqual(force_bytes(inst.meta.tag[0].code), force_bytes("HTEST"))
        self.assertEqual(force_bytes(inst.meta.tag[0].display), force_bytes("test health data"))
        self.assertEqual(force_bytes(inst.meta.tag[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/v3-ActReason"))
        self.assertEqual(inst.term[0].asset[0].period[0].start.date, FHIRDate("2017-06-01").date)
        self.assertEqual(inst.term[0].asset[0].period[0].start.as_json(), "2017-06-01")
        self.assertEqual(force_bytes(inst.term[0].asset[0].subtype[0].text), force_bytes("sample"))
        self.assertEqual(force_bytes(inst.term[0].asset[0].type[0].coding[0].code), force_bytes("RicardianContract"))
        self.assertEqual(force_bytes(inst.term[0].asset[0].type[0].coding[0].system), force_bytes("urn:ietf:rfc:3986"))
        self.assertEqual(inst.term[0].asset[0].valuedItem[0].effectiveTime.date, FHIRDate("1995").date)
        self.assertEqual(inst.term[0].asset[0].valuedItem[0].effectiveTime.as_json(), "1995")
        self.assertEqual(force_bytes(inst.term[0].asset[0].valuedItem[0].entityCodeableConcept.text), force_bytes("Ford Bobcat"))
        self.assertEqual(inst.term[0].asset[0].valuedItem[0].factor, 1.0)
        self.assertEqual(force_bytes(inst.term[0].asset[0].valuedItem[0].identifier.system), force_bytes("http://somewhere.motor-vehicle.com/vin"))
        self.assertEqual(force_bytes(inst.term[0].asset[0].valuedItem[0].identifier.value), force_bytes("XXSVT34-7665t952236"))
        self.assertEqual(force_bytes(inst.term[0].asset[0].valuedItem[0].net.currency), force_bytes("CAD"))
        self.assertEqual(inst.term[0].asset[0].valuedItem[0].net.value, 200.0)
        self.assertEqual(inst.term[0].asset[0].valuedItem[0].points, 1.0)
        self.assertEqual(inst.term[0].asset[0].valuedItem[0].quantity.value, 1)
        self.assertEqual(force_bytes(inst.term[0].asset[0].valuedItem[0].unitPrice.currency), force_bytes("CAD"))
        self.assertEqual(inst.term[0].asset[0].valuedItem[0].unitPrice.value, 200.0)
        self.assertEqual(force_bytes(inst.term[0].group[0].offer.text), force_bytes("Eligible Providers"))
        self.assertEqual(force_bytes(inst.term[0].group[1].offer.text), force_bytes("Responsibility for Payment"))
        self.assertEqual(force_bytes(inst.term[0].group[2].group[0].group[0].offer.text), force_bytes("Emergency Room Copay"))
        self.assertEqual(force_bytes(inst.term[0].group[2].group[0].group[1].offer.text), force_bytes("Professional Visit Copay"))
        self.assertEqual(force_bytes(inst.term[0].group[2].group[0].offer.text), force_bytes("Copays"))
        self.assertEqual(force_bytes(inst.term[0].group[2].group[1].offer.text), force_bytes("Calendar Year Deductible"))
        self.assertEqual(force_bytes(inst.term[0].group[2].group[2].offer.text), force_bytes("Out-Of-Pocket Maximum"))
        self.assertEqual(force_bytes(inst.term[0].group[2].group[3].group[0].offer.text), force_bytes("Ambulance Services"))
        self.assertEqual(force_bytes(inst.term[0].group[2].group[3].group[1].offer.text), force_bytes("Dental Services"))
        self.assertEqual(force_bytes(inst.term[0].group[2].group[3].group[2].offer.text), force_bytes("Diagnostic Services"))
        self.assertEqual(force_bytes(inst.term[0].group[2].group[3].group[3].offer.text), force_bytes("Emergency Room Services"))
        self.assertEqual(force_bytes(inst.term[0].group[2].group[3].group[4].offer.text), force_bytes("Hospital Inpatient Care"))
        self.assertEqual(force_bytes(inst.term[0].group[2].group[3].offer.text), force_bytes("Medical Services"))
        self.assertEqual(force_bytes(inst.term[0].group[2].offer.text), force_bytes("List of Benefits"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.type.coding[0].code), force_bytes("healthinsurance"))
        self.assertEqual(force_bytes(inst.type.coding[0].display), force_bytes("Health Insurance"))
        self.assertEqual(force_bytes(inst.type.coding[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/contract-type"))
    
    def testContract6(self):
        inst = self.instantiate_from("pcd-example-notAuthor.json")
        self.assertIsNotNone(inst, "Must have instantiated a Contract instance")
        self.implContract6(inst)
        
        js = inst.as_json()
        self.assertEqual("Contract", js["resourceType"])
        inst2 = contract.Contract(js)
        self.implContract6(inst2)
    
    def implContract6(self, inst):
        self.assertEqual(force_bytes(inst.friendly[0].contentAttachment.title), force_bytes("The terms of the consent in friendly consumer speak."))
        self.assertEqual(force_bytes(inst.id), force_bytes("pcd-example-notAuthor"))
        self.assertEqual(inst.issued.date, FHIRDate("2015-11-18").date)
        self.assertEqual(inst.issued.as_json(), "2015-11-18")
        self.assertEqual(force_bytes(inst.legal[0].contentAttachment.title), force_bytes("The terms of the consent in lawyer speak."))
        self.assertEqual(force_bytes(inst.meta.tag[0].code), force_bytes("HTEST"))
        self.assertEqual(force_bytes(inst.meta.tag[0].display), force_bytes("test health data"))
        self.assertEqual(force_bytes(inst.meta.tag[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/v3-ActReason"))
        self.assertEqual(force_bytes(inst.subType[0].coding[0].code), force_bytes("Opt-In"))
        self.assertEqual(force_bytes(inst.subType[0].coding[0].display), force_bytes("Default Authorization with exceptions."))
        self.assertEqual(force_bytes(inst.subType[0].coding[0].system), force_bytes("http://www.infoway-inforoute.ca.org/Consent-subtype-codes"))
        self.assertEqual(force_bytes(inst.term[0].offer.text), force_bytes("Withhold all data authored by Good Health provider."))
        self.assertEqual(force_bytes(inst.term[0].type.coding[0].code), force_bytes("withhold-authored-by"))
        self.assertEqual(force_bytes(inst.term[0].type.coding[0].display), force_bytes("Withhold all data authored by specified actor entity."))
        self.assertEqual(force_bytes(inst.term[0].type.coding[0].system), force_bytes("http://example.org/fhir/consent-term-type-codes"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.type.coding[0].code), force_bytes("57016-8"))
        self.assertEqual(force_bytes(inst.type.coding[0].system), force_bytes("http://loinc.org"))
    
    def testContract7(self):
        inst = self.instantiate_from("pcd-example-notThem.json")
        self.assertIsNotNone(inst, "Must have instantiated a Contract instance")
        self.implContract7(inst)
        
        js = inst.as_json()
        self.assertEqual("Contract", js["resourceType"])
        inst2 = contract.Contract(js)
        self.implContract7(inst2)
    
    def implContract7(self, inst):
        self.assertEqual(force_bytes(inst.friendly[0].contentAttachment.title), force_bytes("The terms of the consent in friendly consumer speak."))
        self.assertEqual(force_bytes(inst.id), force_bytes("pcd-example-notThem"))
        self.assertEqual(inst.issued.date, FHIRDate("2015-11-18").date)
        self.assertEqual(inst.issued.as_json(), "2015-11-18")
        self.assertEqual(force_bytes(inst.legal[0].contentAttachment.title), force_bytes("The terms of the consent in lawyer speak."))
        self.assertEqual(force_bytes(inst.meta.tag[0].code), force_bytes("HTEST"))
        self.assertEqual(force_bytes(inst.meta.tag[0].display), force_bytes("test health data"))
        self.assertEqual(force_bytes(inst.meta.tag[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/v3-ActReason"))
        self.assertEqual(force_bytes(inst.signer[0].signature[0].type[0].code), force_bytes("1.2.840.10065.1.12.1.1"))
        self.assertEqual(force_bytes(inst.signer[0].signature[0].type[0].system), force_bytes("urn:iso-astm:E1762-95:2013"))
        self.assertEqual(inst.signer[0].signature[0].when.date, FHIRDate("2013-06-08T10:57:34-07:00").date)
        self.assertEqual(inst.signer[0].signature[0].when.as_json(), "2013-06-08T10:57:34-07:00")
        self.assertEqual(force_bytes(inst.signer[0].type.code), force_bytes("COVPTY"))
        self.assertEqual(force_bytes(inst.signer[0].type.system), force_bytes("http://terminology.hl7.org/CodeSystem/contractsignertypecodes"))
        self.assertEqual(force_bytes(inst.subType[0].coding[0].code), force_bytes("Opt-In"))
        self.assertEqual(force_bytes(inst.subType[0].coding[0].display), force_bytes("Default Authorization with exceptions."))
        self.assertEqual(force_bytes(inst.subType[0].coding[0].system), force_bytes("http://www.infoway-inforoute.ca.org/Consent-subtype-codes"))
        self.assertEqual(force_bytes(inst.term[0].offer.text), force_bytes("Withhold this order and any results or related objects from specified nurse provider."))
        self.assertEqual(force_bytes(inst.term[0].type.coding[0].code), force_bytes("withhold-from"))
        self.assertEqual(force_bytes(inst.term[0].type.coding[0].display), force_bytes("Withhold all data from specified actor entity."))
        self.assertEqual(force_bytes(inst.term[0].type.coding[0].system), force_bytes("http://example.org/fhir/consent-term-type-codes"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.type.coding[0].code), force_bytes("57016-8"))
        self.assertEqual(force_bytes(inst.type.coding[0].system), force_bytes("http://loinc.org"))
    
    def testContract8(self):
        inst = self.instantiate_from("pcd-example-notThis.json")
        self.assertIsNotNone(inst, "Must have instantiated a Contract instance")
        self.implContract8(inst)
        
        js = inst.as_json()
        self.assertEqual("Contract", js["resourceType"])
        inst2 = contract.Contract(js)
        self.implContract8(inst2)
    
    def implContract8(self, inst):
        self.assertEqual(force_bytes(inst.friendly[0].contentAttachment.title), force_bytes("The terms of the consent in friendly consumer speak."))
        self.assertEqual(force_bytes(inst.id), force_bytes("pcd-example-notThis"))
        self.assertEqual(inst.issued.date, FHIRDate("2015-11-18").date)
        self.assertEqual(inst.issued.as_json(), "2015-11-18")
        self.assertEqual(force_bytes(inst.legal[0].contentAttachment.title), force_bytes("The terms of the consent in lawyer speak."))
        self.assertEqual(force_bytes(inst.meta.tag[0].code), force_bytes("HTEST"))
        self.assertEqual(force_bytes(inst.meta.tag[0].display), force_bytes("test health data"))
        self.assertEqual(force_bytes(inst.meta.tag[0].system), force_bytes("http://terminology.hl7.org/CodeSystem/v3-ActReason"))
        self.assertEqual(force_bytes(inst.subType[0].coding[0].code), force_bytes("Opt-In"))
        self.assertEqual(force_bytes(inst.subType[0].coding[0].display), force_bytes("Default Authorization with exceptions."))
        self.assertEqual(force_bytes(inst.subType[0].coding[0].system), force_bytes("http://www.infoway-inforoute.ca.org/Consent-subtype-codes"))
        self.assertEqual(inst.term[0].applies.start.date, FHIRDate("2015-11-18").date)
        self.assertEqual(inst.term[0].applies.start.as_json(), "2015-11-18")
        self.assertEqual(force_bytes(inst.term[0].identifier.system), force_bytes("http://example.org/fhir/term-items"))
        self.assertEqual(force_bytes(inst.term[0].identifier.value), force_bytes("3347689"))
        self.assertEqual(inst.term[0].issued.date, FHIRDate("2015-11-01").date)
        self.assertEqual(inst.term[0].issued.as_json(), "2015-11-01")
        self.assertEqual(force_bytes(inst.term[0].offer.text), force_bytes("Withhold this order and any results or related objects from any provider."))
        self.assertEqual(force_bytes(inst.term[0].type.coding[0].code), force_bytes("withhold-identified-object-and-related"))
        self.assertEqual(force_bytes(inst.term[0].type.coding[0].display), force_bytes("Withhold the identified object and any other resources that are related to this object."))
        self.assertEqual(force_bytes(inst.term[0].type.coding[0].system), force_bytes("http://example.org/fhir/consent-term-type-codes"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.type.coding[0].code), force_bytes("57016-8"))
        self.assertEqual(force_bytes(inst.type.coding[0].system), force_bytes("http://loinc.org"))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.0-a53ec6ee1b (http://hl7.org/fhir/StructureDefinition/Expression) on 2019-05-13.
#  2019, SMART Health IT.


from . import element

class Expression(element.Element):
    """ An expression that can be used to generate a value.

    A expression that is evaluated in a specified context and returns a value.
    The context of use of the expression must specify the context in which the
    expression is evaluated, and how the result of the expression is used.
    """

    resource_type = "Expression"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.description = None
        """ Natural language description of the condition.
        Type `str`. """

        self.expression = None
        """ Expression in specified language.
        Type `str`. """

        self.language = None
        """ text/cql | text/fhirpath | application/x-fhir-query | etc..
        Type `str`. """

        self.name = None
        """ Short name assigned to expression for reuse.
        Type `str`. """

        self.reference = None
        """ Where the expression is found.
        Type `str`. """

        super(Expression, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(Expression, self).elementProperties()
        js.extend([
            ("description", "description", str, "string", False, None, False),
            ("expression", "expression", str, "string", False, None, False),
            ("language", "language", str, "code", False, None, True),
            ("name", "name", str, "id", False, None, False),
            ("reference", "reference", str, "uri", False, None, False),
        ])
        return js


